"""Module providing the class Polynomial and low-level functions"""

from sympy import Basic, Symbol, Pow, Number, Add, Mul, Integer, Rational, \
        sympify, symbols, all
# Use (faster) Singleton comparisons.
from sympy.core.basic import S
# Need numbers.gcd, for content.
from sympy.core.numbers import igcd
# To determine coefficient type:
from sympy.core.numbers import NumberSymbol, ImaginaryUnit

from sympy.polynomials import fast
import sympy.polynomials    # for sympy.polynomials.wrapper  (cyclic)

# This is the list of possible rings the coefficients could lie in,
# ordered by inclusion.
coeff_rings = ['int', 'rat', 'real', 'cplx', 'sym']


# Global default, probably the most efficient for division?
default_order = 'grevlex'


# Local exception type.
class PolynomialException(Exception):
    pass


class Polynomial(Basic):
    """Unified representation of all kinds of polynomials.

    Usage:
        Most of the time, the Polynomial instances are created of a
        SymPy expression, which is a polynomial.

        Optionally, the user can give a 'var' argument containing the
        list of variables (in order), so that he can view 2*x + x*y as
        a polynomial in x only, for example. If not given, the
        occuring symbols are extracted automatically and sorted
        alphabetically.

        The (optional) argument 'order' defines the monomial order,
        which is of importance to division algorithms and Groebner
        bases and defaults to 'grevlex', that is graded reverse
        lexicographic ordering. Other options are:
        'lex' - lexicographic order
        'grlex' - graded lexicographic order
        '1-el' - first elimination order

        Alternatively, a Polynomial can be instantiated by giving its
        coefficients and exponents in nested tuples, alone or in
        addidition to the sympy_expr. Here, no consistency checks are
        done as this mode is intended for internal use mostly.

        The built Polynomial instances support arithmetic, like
        addition and multiplication, but fall back to the underlying
        SymPy expression, when a non-Polynomial is encountered.

        The SymPy expression of a Polynomial f can be accessed through
        f.sympy_expr. The coefficients and exponents are held in
        f.coeffs , f.var and f.order hold the respective arguments.

    Notes:
        Computes the coefficients with the exponents in sparse
        representation for faster arithmetic and leading terms etc.
        Tries to be compatible with other SymPy expressions, for
        example, by forwarding most attributes like assumptions to the
        underlying SymPy expression.

    Examples:
        >>> from sympy import *
        >>> x, y = symbols('xy')
        >>> f = Polynomial(x + 1)
        >>> f.sympy_expr
        1 + x
        >>> f.coeffs
        ((1, 1), (1, 0))
        >>> f.var
        [x]
        >>> f.order
        'grevlex'
        >>> print f
        1 + x
        >>> print srepr(f)
        Polynomial(Add(One, Symbol('x')), ((One, One), (One, Zero)), [Symbol('x')], 'grevlex')
        >>> g = Polynomial(y**2 - x*y)
        >>> s = f + g
        >>> s.var == [x, y]
        True
        >>> bool(s == y**2 - x*y + x + 1)
        True
        >>> h = Polynomial(g.sympy_expr, var=y)
        >>> g.coeffs
        ((-1, 1, 1), (1, 0, 2))
        >>> h.coeffs
        ((1, 2), (-x, 1))

    Also see L{sympy2coefficients}, L{coefficients2sympy}.

    """

    def __new__(cls, sympy_expr=None, coeffs=None, var=None, order=None,
                **assumptions):
        obj = Basic.__new__(cls)

        if sympy_expr is None and coeffs is None:
            raise PolynomialException("No polynomial data given!")
        elif sympy_expr is not None and coeffs is None:
            # Polynomial is contructed by the sympy expression.
            sympy_expr = sympify(sympy_expr).expand()
            if var is None:
                # Automatically extract the variables from the expression.
                var = list(sympy_expr.atoms(Symbol))
                var.sort()
            if isinstance(var, Symbol):
                var = [var]
            elif isinstance(var, tuple):
                var = list(var)
            obj.var = var
            if var and not sympy_expr.is_polynomial(*var):
                raise PolynomialException("%s is not a polynomial!"
                                          % sympy_expr)
            obj.sympy_expr = sympy_expr
            if order is None:
                order = default_order
            obj.order = order
            obj.coeffs = sympy2coefficients(sympy_expr, var, order)
        elif sympy_expr is None and coeffs is not None:
            # This polynomial is constructed by its coeffs.
            # No sympify is used, all Terms are assumed to be
            # instances of Basic.
            obj.coeffs = coeffs
            if var is None:
                raise PolynomialException("Ambiguous data, "
                                          + "please specify the var.")
            if isinstance(var, Symbol):
                var = [var]
            obj.var = var
            if order is None:
                order = default_order
            obj.order = order
            obj.sympy_expr = coefficients2sympy(coeffs, var)
        else:
            # Both the sympy expression and the coeffs are given.
            # No sanity check done, which would be slower than just
            # giving one of them.
            obj.sympy_expr = sympy_expr
            obj.coeffs = coeffs
            if var is None:
                var = list(sympy_expr.atoms(Symbol))
                var.sort()
            if isinstance(var, Symbol):
                var = [var]
            obj.var = var
            if order is None:
                order = default_order
            obj.order = order
        return obj


    def __getattribute__(self, name):
        """Redirect most attributes to the underlying SymPy expression."""

        # Check if the attribute belongs to the Polynomial itself:
        if name not in ("__class__",
                        "__dict__",
                        "__new__",
                        "__getattribute__",
                        "__str__",
                        "__repr__",
                        "__eq__",
                        "__ne__",
                        "__pos__",
                        "__neg__",
                        "__add__",
                        "__radd__",
                        "__sub__",
                        "__rsub__",
                        "__mul__",
                        "__rmul__",
                        "__call__",
                        "as_integer",
                        "as_monic",
                        "as_primitive",
                        "coeff",
                        "content",
                        "diff",
                        "integrate",
                        "leading_coeff",
                        "leading_term",
                        "nth_coeff",
                        "coeffs",
                        "order",
                        "degree",
                        "as_basic",
                        "sympy_expr",
                        "var"):
            try:
                # This fails when the Polynomial is instantiated.
                se = object.__getattribute__(self, "__dict__")["sympy_expr"]
                # This uses the SymPy expressions' attributes
                return object.__getattribute__(se, name)
            except KeyError:
                # The .sympy_expr doesn't yet exist.
                pass

        # This uses the Polynomial's attributes
        return object.__getattribute__(self, name)

    def __eq__(self, other):
        """Equality is restricted to equality of SymPy expression.

        This overwrites the == operator. Other attributes such as
        variables or monomial order are not compared.

        """

        if isinstance(other, Polynomial):
            return self.sympy_expr == other.sympy_expr
        else:
            return self.sympy_expr == other


    def __ne__(self, other):
        """Also see L{__eq__}."""
        if isinstance(other, Polynomial):
            return self.sympy_expr != other.sympy_expr
        else:
            return self.sympy_expr != other


    def __pos__(self):
        """Just returns the Polynomial."""
        return self


    def __neg__(self):
        """Returns the Polynomial multiplied by -1."""
        return Polynomial(sympy_expr=-self.sympy_expr,
                          coeffs=tuple([(-term[0],) + term[1:]
                                        for term in self.coeffs]),
                          var=self.var,
                          order=self.order)


    def __add__(self, other):
        """Overwrites the + operator.

        Implements an addition algorithm for instances of Polynomial
        with matching variables, using the coefficients and exponents,
        but falls back to the SymPy expressions otherwise. It even
        returns a non-Polynomial object when encountering one.

        Also see L{Polynomial}, L{__mul__}.

        """

        # Fall back to sympy expression, when other is no Polynomial.
        if not isinstance(other, Polynomial):
            return self.sympy_expr + other

        # When coeffs are not compatible, use the sum of the
        # SymPy expressions.
        if self.var != other.var \
           or (len(self.var) > 1 and self.order != other.order):
            return Polynomial(self.sympy_expr + other.sympy_expr)

        # Check if one is 0, then return the other.
        if self.sympy_expr is S.Zero:
            return other
        if other.sympy_expr is S.Zero:
            return self

        # Now we are going to do the addition using the coeffs.
        # Merge the terms of self and other:
        result_list = []
        s, o = self.coeffs, other.coeffs
        i, j = 0, 0
        while i < len(s) and j < len(o):
            if (s[i][1:] == o[j][1:]):
                c = s[i][0] + o[j][0]
                if c is not S.Zero:
                    result_list.append((c,) + s[i][1:])
                i += 1
                j += 1
            elif term_cmp(s[i], o[j], self.order) > 0:
                result_list.append(s[i])
                i += 1
            else:
                result_list.append(o[j])
                j += 1
        # Append the rest.
        result_list += s[i:]
        result_list += o[j:]
        # Check if something was appended to the (empty) result.
        if len(result_list) == 0:
            return Polynomial(S.Zero, var=self.var, order=self.order)

        return Polynomial(coeffs=tuple(result_list), var=self.var,
                          order=self.order)


    def __radd__(self, other):
        """Also see L{__add__}."""
        return self.__add__(other)


    def __sub__(self, other):
        """Also see L{__add__},  L{__neg__}."""
        return self.__add__(-other)


    def __rsub__(self, other):
        """Also see L{__add__}, L{__neg__}"""
        return (-self).__add__(other)


    def __mul__(self, other):
        """Overwrites the * operator.

        Implements a multiplication algorithm for instances of Polynomial
        with matching variables, using the coefficients and exponents,
        but falls back to the SymPy expressions otherwise. It even
        returns a non-Polynomial object when encountering one.

        Also see L{Polynomial}, L{__add__}.

        """

        # Fall back to SymPy expression, if other is no Polynomial.
        if not isinstance(other, Polynomial):
            return self.sympy_expr * other

        # When coeffs are not compatible, use the product of the
        # SymPy expressions.
        if self.var != other.var \
               or (len(self.var) > 1 and self.order != other.order):
            return Polynomial(self.sympy_expr * other.sympy_expr)

        # Check if one is 0, then return 0.x
        if self.sympy_expr is S.Zero:
            return self
        if other.sympy_expr is S.Zero:
            return other

        # Check if one is 1, then return other.
        if self.sympy_expr is S.One:
            return other
        if other.sympy_expr is S.One:
            return self

        # Now we are going to do the multiplication using the coefficients.
        result_dict = {}
        for self_term in self.coeffs:
            for other_term in other.coeffs:
                key = tuple([i+j for i,j in zip(self_term[1:], other_term[1:])])
                if result_dict.has_key(key):
                    result_dict[key] += (self_term[0]*other_term[0]).expand()
                else:
                    result_dict[key] = (self_term[0]*other_term[0]).expand()
        result_list = [(result_dict[key],) + key for key in result_dict
                       if result_dict[key] is not S.Zero]
        result_list.sort(cmp=lambda x,y:term_cmp(x,y,self.order), reverse=True)

        return Polynomial(coeffs=tuple(result_list),
                          var=self.var,
                          order=self.order)


    def __rmul__(self, other):
        """Also see L{__mul__}."""
        return self.__mul__(other)


    def __pow__(self, other):
        """Overwrites the ** operator."""
        if isinstance(other, Polynomial):
            other = other.sympy_expr
        if other is S.Zero:
            return Polynomial(S.One, var=self.var, order=self.order)
        elif other is S.One:
            return self
        elif isinstance(other, Integer) and other.is_positive:
            # TODO: Implement efficient power algorithm using coeffs?
            return Polynomial(self.sympy_expr**other,
                              var=self.var, order=self.order)
        else:
            # Probably not a polynomial, fall back to sympy expression.
            return self.sympy_expr**other


    def __call__(self, *point):
        """Evaluate the polynomial function at a specific point.

        Usage:
            Give an arbitrary argument for each variable and get the
            SymPy expression with the variables substituted.

        Examples:
            >>> x, y = symbols('xy')
            >>> f = Polynomial(2*x - y)
            >>> f(1, 7)
            -5
            >>> f(3*x, x)
            5*x

        """

        if len(point) != len(self.var):
            raise PolynomialException('No proper input for evaluation.')
        result = self.sympy_expr
        for v, x in zip(self.var, point):
            result = result.subs(v, x)
        return result


    def as_integer(self):
        """Return the polynomial with integer coefficients.

        Usage:
            Starting from an instance of Polynomial with only rational
            coefficients, this function multiplies it with the common
            denominator. The result is a tuple consisting of the
            factor applied to the coefficients and a new instance of
            Polynomial in the integers.

        Example:
            >>> x, y = symbols('xy')
            >>> f = Polynomial(x/6 + y/4 + x*y/3)
            >>> denominator, f = f.as_integer()
            >>> print denominator
            12
            >>> print f
            2*x + 3*y + 4*x*y
            >>> denominator, f = f.as_integer()
            >>> print denominator
            1
            >>> print f
            2*x + 3*y + 4*x*y

        Also see L{as_monic}, L{as_primitive}.

        """

        denom = S.One
        for term in self.coeffs:
            if not isinstance(term[0], Rational):
                raise PolynomialException("%s is no rational coefficient!" % term[0])
            else:
                # Compute the least common multiple of the denominators:
                denom = term[0].q*denom/igcd(int(denom), int(term[0].q))
        if denom is S.One:
            return S.One, self
        else:
            return (denom,
                    Polynomial(coeffs=tuple([((term[0]*denom),) + term[1:]
                                             for term in self.coeffs]),
                               var=self.var, order=self.order))


    def as_monic(self):
        """Return the polynomial with leading coefficient 1.

        Usage:
            Starting with any instance of Polynomial, this returns the
            former leading coefficient and a new Polynomial which is
            monic.

        Examples:
            >>> x, y = symbols('xy')
            >>> f = Polynomial(x/2 + y/4 + x*y/3)
            >>> leadcoeff, f = f.as_monic()
            >>> print leadcoeff
            1/3
            >>> print f
            3*x/2 + 3*y/4 + x*y
            >>> leadcoeff, f = f.as_monic()
            >>> print leadcoeff
            1
            >>> print f
            3*x/2 + 3*y/4 + x*y

            >>> f = Polynomial(y*x, var=x)
            >>> leadcoeff, f = f.as_monic()
            >>> print leadcoeff
            y
            >>> print f
            x

        Also see L{as_integer}, L{as_primitive}, L{leading_coeff}.

        """

        lc = self.leading_coeff()
        return lc, Polynomial(coeffs=tuple([((term[0]/lc).expand(),) + term[1:]
                                            for term in self.coeffs]),
                              var=self.var, order=self.order)


    def as_primitive(self):
        """Return the content and a primitive Polynomial.

        Usage:
            Starting with any instance of Polynomial, this returns the
            content, that is, the greatest common divisor of the
            (integer-or-symbolic) coefficients, and a new Polynomial which is
            primitive, that is, of content 1. Only works for integer
            coefficients.

        Examples:
            >>> x, y = symbols('xy')
            >>> f = Polynomial(6*x + 20*y + 4*x*y)
            >>> content, f = f.as_primitive()
            >>> print content
            2
            >>> print f
            3*x + 10*y + 2*x*y
            >>> content, f = f.as_primitive()
            >>> print content
            1
            >>> print f
            3*x + 10*y + 2*x*y

        Also see L{as_integer}, L{as_monic}, L{content}.

        """

        c = self.content()
        if self.coeffs[0][0] < 0:
            sign = -1
        else:
            sign = 1

        if c is S.Zero:
            return S.Zero, self
        if c is S.One and sign == 1:
            return S.One, self
        else:
            c *= sign
            return c, Polynomial(coeffs=tuple([(term[0]/c,) + term[1:]
                                               for term in self.coeffs]),
                                 var=self.var, order=self.order)


    def content(self):
        """Return the content of a Polynomial.

        Usage:
            Returns the content, that is, the positive greatest common
            divisor of the (integer-or-symbolic) coefficients.

        Examples:
            >>> x, y = symbols('xy')
            >>> f = Polynomial(6*x + 20*y + 4*x*y)
            >>> f.content()
            2
            >>> f = Polynomial(y**2*x**2 + y, var=x)
            >>> f.content()
            y

        Also see L{as_primitive}, L{leading_coeff}.

        """

        result_int = 0  # part result for integer coeffs
        result_sym = 0  # part result for symbolic coeffs
        for term in self.coeffs:
            if isinstance(term[0], Integer):
                result_int = abs(igcd(result_int, abs(int(term[0]))))
            elif isinstance(term[0], Number):
                raise PolynomialException("%s is no integer coefficient!" % term[0])
            else:
                # XXX this will not work for sin(x), sqrt(2), etc...
                result_sym = sympy.polynomials.wrapper.gcd(result_sym, term[0])

        if result_sym == 0:
            # only int coeffs -- return Integer
            return Integer(result_int)
        else:
            return sympy.polynomials.wrapper.gcd(result_sym, result_int)


    def diff(self, variable):
        """Derivative of a Polynomial.

        Usage:
            Returns a new instance of Polynomial which is the partial
            derivative by the given variable.

        Examples:
            >>> x, y, z = symbols('xyz')
            >>> f = Polynomial(6*x + 20*y + 4*x*y)
            >>> fx = f.diff(x)
            >>> print fx
            6 + 4*y
            >>> fz = f.diff(z)
            >>> print fz
            0

        """

        if not variable in self.var:
            return Polynomial(S.Zero, var=self.var, order=self.order)
        for i, v in enumerate(self.var):
            if v is variable:
                i += 1 # Variables begin at index 1 in coeffs.
                break
        result_list = [((term[0]*term[i]).expand(),) + term[1:i]
                       + ((term[i]-1),) + term[i+1:]
                       for term in self.coeffs if term[i].is_positive]
        if len(result_list) == 0:
            return Polynomial(sympy_expr=S.Zero, var=self.var,
                              order=self.order)
        else:
            result_list.sort(cmp=lambda x,y:term_cmp(x,y,self.order),
                             reverse=True)
            return Polynomial(coeffs=tuple(result_list), var=self.var,
                              order=self.order)

    def integrate(self, variable):
        """Primitive function of a Polynomial.

        Usage:
            Returns a new instance of Polynomial which is the primitive
            function (antiderivative) of "self" with respect to the given
            variable.

        Examples:
            >>> x, y, z = symbols('xyz')
            >>> f = Polynomial(6*x + 20*y + 4*x*y)
            >>> fx = f.integrate(x)
            >>> print fx
            20*x*y + 3*x**2 + 2*y*x**2
            >>> fz = f.integrate(z)
            >>> print fz
            6*x*z + 20*y*z + 4*x*y*z

        """

        if not variable in self.var:
            # FIXME we have to ensure first that either:
            # - coeff_ring != 'sym'
            # - coefficients do not contain `variable`
            return Polynomial(variable, order=self.order) * self
        nvar = self.var.index(variable)
        cint = []
        for term in self.coeffs:
            t = list(term)
            t[nvar+1] += 1
            t[0] /= t[nvar+1]
            cint.append(tuple(t))

        return Polynomial(coeffs=cint, var=self.var, order=self.order)

    def coeff(self, x, n):
        """Returns the coefficient at x**n

        Example:
        >>> a, x = symbols("ax")
        >>> f = (a+1)*x + (a+2)*x**2 + a
        >>> Polynomial(f).coeff(x, 2)
        2 + a
        >>> Polynomial(f).coeff(a, 1)
        1 + x + x**2
        """
        f = Polynomial(self, var=[x])
        return f.nth_coeff(n)


    def leading_coeff(self):
        """Return the leading coefficient of a Polynomial.

        Usage:
            This gives the coefficient, that is, non-symbolic part, of
            the leading term, according to the monomial order, or
            simply highest degree, in the univariate case.

        Examples:
            >>> x, y = symbols('xy')
            >>> f = Polynomial(6*x + 20*y + 4*x*y)
            >>> f.leading_coeff()
            4

        Also see L{as_monic}, L{leading_term}, L{nth_coeff}.

        """

        return self.coeffs[0][0]


    def leading_term(self):
        """Return the leading term of a Polynomial.

        Usage:
            The leading term, according to the monomial order, or
            simply highest degree, in the univariate case.

        Examples:
            >>> x, y = symbols('xy')
            >>> f = Polynomial(6*x + 20*y + 4*x*y)
            >>> print f.leading_term()
            4*x*y

        Also see L{leading_coeff}.

        """

        return Polynomial(coeffs=(self.coeffs[0],), var=self.var,
                          order=self.order)


    def nth_coeff(self, *exponent):
        """Return a specific coefficient of a Polynomial.

        Usage:
            This gives the coefficient, that is, non-symbolic part, of
            the term with matching exponents, or 0, if it doesn't appear.

        Examples:
            >>> x, y = symbols('xy')
            >>> f = Polynomial(6*x + 20*y + 4*x*y)
            >>> f.nth_coeff(1, 0)
            6
            >>> f.nth_coeff(1, 1)
            4
            >>> f.nth_coeff(0, 0)
            0

        Also see L{leading_coeff}.

        """

        for term in self.coeffs:
            if term[1:] == exponent:
                return term[0]
        else: # No term with matching exponent found.
            return S.Zero

    def degree(self):
        return sum(self.coeffs[0][1:])

    def as_basic(self):
        return self.sympy_expr

def sympy2coefficients(sympy_expr, var, order):
    """Return the tuple of coefficients and exponents.

    Usage:
        This functions computes the tuples of coefficients and
        exponents from a given SymPy expression. This expression is
        assumed to be expanded already. The arguments 'var' and
        'order' define the occuring variables and the monomial order
        to use, respectively.

        Normally, the user would never call this function himself, it
        is rather used within the Polynomial class.

    Examples:
        >>> x, y = symbols('xy')
        >>> sympy2coefficients(2*x + 3, [x], 'lex')
        ((2, 1), (3, 0))
        >>> sympy2coefficients(x**2*y + 4*y, [x, y], 'lex')
        ((1, 2, 1), (4, 0, 1))
        >>> sympy2coefficients(x**2*y + 4*y, [y], 'lex')
        ((4 + x**2, 1),)

    Also see L{Polynomial}, L{sympy2coefficients}.

    """

    result_dict = {}
    if sympy_expr.is_Add:
        terms = sympy_expr.args[:]
    else:
        terms = [sympy_expr]
    for term in terms:
        if term.is_Mul:
            factors = term.args[:]
        else:
            factors = [term]
        c = S.One
        exponents = [S.Zero]*len(var)
        for factor in factors:
            # Check if any of the variables occur.
            if filter(lambda x:x in var, factor.atoms(Symbol)):
                if isinstance(factor, Pow) \
                   and isinstance(factor.base, Symbol) \
                   and factor.exp.is_integer \
                   and factor.exp.is_positive:
                    exponents[var.index(factor.base)] += factor.exp
                elif isinstance(factor, Symbol):
                    exponents[var.index(factor)] += 1
                else:
                    raise PolynomialException("%s is not a polynomial!"
                                              % sympy_expr)
            else: # The factor is relativly constant.
                c *= factor
        exponents = tuple(exponents)
        if result_dict.has_key(exponents):
            result_dict[exponents] += c
        else:
            result_dict[exponents] = c

    coefficient_list = [(result_dict[key],) + key
                        for key in result_dict.keys()
                        if result_dict[key] is not S.Zero]
    coefficient_list.sort(cmp=lambda x,y:term_cmp(x, y, order), reverse=True)
    if len(coefficient_list) == 0:
        coefficient_list = [tuple([S.Zero]*(len(var) + 1))]
    return tuple(coefficient_list)


def coefficients2sympy(coeffs, var):
    """Return the SymPy expression of given coefficients and exponents.

    Usage:
        This functions computes the original SymPy expression from the
        tuples of coefficients and exponents. The argument 'var'
        defines the occuring variables.

        Normally, the user would never call this function himself, it
        is rather used within the Polynomial class.

    Examples:
        >>> x, y = symbols('xy')
        >>> coefficients2sympy(((2, 1), (3, 0)), [x])
        3 + 2*x
        >>> coefficients2sympy(((1, 2, 1), (4, 0, 1)), [x, y])
        4*y + y*x**2
        >>> coefficients2sympy(((1, 2, 1), (4, 0, 1)), [y, x])
        4*x + x*y**2

    Also see L{Polynomial}, L{sympy2coefficients}.

    """

    if len(coeffs) == 0:
        raise PolynomialException('Bad coefficient list.')
    elif len(coeffs[0]) != len(var) + 1:
        raise PolynomialException('Wrong number of var given.')

    result = S.Zero
    for term in coeffs:
        c = term[0]
        for i, v in enumerate(var):
            c *= v**term[i+1]
        result += c
    return result


# Simple helper functions common to several algorithms.

def reverse(t):
    """Return a tuple with reversed order"""
    return tuple([t[i] for i in range(len(t)-1, 0, -1)])


def term_cmp(a, b, order):
    """Compares tuples occuring in the Polynomial's coeffs."""
    if order == 'lex':
        return cmp(a[1:], b[1:])
    elif order == 'grlex':
        return cmp((sum(a[1:]),) + a[1:], (sum(b[1:]),) + b[1:])
    elif order == 'grevlex':
        return cmp((sum(a[1:]),) + reverse(map(lambda l:-l, a[1:])),
                   (sum(b[1:]),) + reverse(map(lambda l:-l, b[1:])))
    elif order == '1-el':
        return cmp((a[1], sum(a[2:])) + reverse(map(lambda l:-l,a[2:])),
                   (b[1], sum(b[2:])) + reverse(map(lambda l:-l,b[2:])))
    else:
        raise PolynomialException(str(order) + 'is not an implemented order.')


def term_mult(a, b):
    """Multiplication of a tuple representing a term.

    a and b are assumed to be tuples of some Polynomial's coeffs of
    same length.

    """

    return ((a[0]*b[0]).expand(),) \
           + tuple(map(lambda (x,y): x+y, zip(a[1:], b[1:])))

def term_div(a, b):
    """Division of a tuple representing a term.

    a and b are assumed to be tuples of some Polynomial's coeffs of
    same length.

    """

    return ((a[0]/b[0]).expand(),) \
           + tuple(map(lambda (x,y): x-y, zip(a[1:], b[1:])))


def term_is_mult(a, b):
    """Return True if a is a multiple of b.

    a and b are assumed to be tuples some Polynomial's coeffs same
    length.

    """

    return all([x.is_nonnegative for x in term_div(a, b)[1:]])


def term_lcm(a, b):
    """Least common multiple of tuples representing terms.

    a and b are assumed to be tuples some Polynomial's coeffs same
    length. The coefficient is set to 1.

    """

    return (S.One,) + tuple([max(aa, bb) for aa, bb in zip(a[1:], b[1:])])


def merge_var(*a):
    """Return a sorted list of the symbols in the arguments"""
    result = []
    for var in a:
        for sym in var:
            if not sym in result:
                result.append(sym)
    result.sort()
    return result


def coeff_ring(atom):
    """Determine the number type of some atom, or some list of atoms."""
    if isinstance(atom, (Number, NumberSymbol, ImaginaryUnit)) \
        or (isinstance(atom, (Add, Mul))
            and all(map(lambda a:isinstance(a, (Number, NumberSymbol,
                                                ImaginaryUnit)), atom.args))) \
        or (isinstance(atom, Pow) and isinstance(atom.base, (Number,
                                                             NumberSymbol,
                                                             ImaginaryUnit))
            and isinstance(atom.exp, (Number, NumberSymbol, ImaginaryUnit))):
        if atom.is_integer:
            return 'int'
        elif atom.is_rational:
            return 'rat'
        elif atom.is_real:
            return 'real'
        else:
            return 'cplx'
    elif isinstance(atom, list):
        # Get the coefficient ring of each atom and look for the worst case.
        result = 'int'
        for a in atom:
            cr = coeff_ring(a)
            assert type(cr) == str
            if coeff_rings.index(cr) > coeff_rings.index(result):
                result = cr
        return result
    else:
        return 'sym'


def get_numbers(atom):
    """Extracts the non-symbolic part of an expression."""
    # TODO: Merge with coeff_ring, without recursion!
    result = []
    if isinstance(atom, (Number, NumberSymbol, ImaginaryUnit)) \
        or (isinstance(atom, (Add, Mul))
            and all(map(lambda a:isinstance(a, (Number,
                                                NumberSymbol,
                                                ImaginaryUnit)),
                        atom.args[:]))) \
        or (isinstance(atom, Pow) and isinstance(atom.base, (Number,
                                                             NumberSymbol,
                                                             ImaginaryUnit))
            and isinstance(atom.exp, (Number, NumberSymbol, ImaginaryUnit))):
        return [atom]
    elif isinstance(atom, (Add, Mul)):
        for a in atom.args:
            result.append(get_numbers(a))
    return result


def integer_divisors(n):
    """Returns a list of all positive integer divisors of n."""
    n = abs(n)
    r = []
    for i in range(1, n/2+1):
        if n % i == 0:
            r.append(i)
    r.append(n)
    return r


# Cooperation with the submodule of faster arithmetic.

def Polynomial2IntPoly(f):
    """Converts an instance of Polynomial to one of IntPoly.

    Nothing is checked.
    """

    result_dict = {}
    for t in f.coeffs:
        if t[0] is not S.Zero:
            assert isinstance(t[0], Integer)
            result_dict[int(t[1])] = int(t[0])
    return fast.intpoly.IntPoly(result_dict)


def IntPoly2Polynomial(f, var, order):
    """Converts an instance of IntPoly to one of Polynomial.

    Nothing is checked.
    """

    coeffs = []
    exponents = f.coeffs.keys()
    exponents.sort(reverse=True)
    for exp in exponents:
        coeffs.append((sympify(f[exp]), sympify(exp)))
    return Polynomial(coeffs=tuple(coeffs), var=var, order=order)



"""
    elliptic.py

    Implements the Jacobi theta and Jacobi elliptic functions, using
    arbitrary precision math library

    Author: M.T. Taschuk

    References:

    [1] Abramowitz & Stegun. 'Handbook of Mathematical Functions, 9th Ed.',
        (Dover duplicate of 1972 edition)
    [2] Whittaker 'A Course of Modern Analysis, 4th Ed.', 1946,
        Cambridge Univeristy Press

"""
import sys

from mptypes import (mpf, convert_lossless, eps)
from functions import (pi, sqrt, cos, sin, exp, ellipk, sech)

def calculate_nome(k):
    """
    Calculate the nome, q, from the value for k.

    Useful factoids:

    k**2 = m;   m is used in Abramowitz
    """
    k = convert_lossless(k)

    if k > mpf('1'):             # range error
        raise ValueError

    zero = mpf('0')
    one = mpf('1')

    if k == zero:
        return zero
    elif k == one:
        return one
    else:
        kprimesquared = one - k**2
        kprime = sqrt(kprimesquared)
        top = ellipk(kprimesquared)
        bottom = ellipk(k**2)

        argument = mpf('-1')*pi*top/bottom

        nome = exp(argument)
        return nome

def calculate_k(q, verbose=False):
    """
    Calculates the value of k for a particular nome, q.

    Uses special cases of the jacobi theta functions, with
    q as an argument, rather than m.

    k = (v2(0, q)/v3(0, q))**2
    """
    zero = mpf('0')
    one = mpf('1')

    q = convert_lossless(q)
    if q > one or q < zero:
        raise ValueError

    # calculate v2(0, q)
    sum = zero
    term = zero                     # series starts at zero
    while True:
        factor1 = q**(term*(term + 1))
        term_n = factor1            # suboptimal, kept for readability
        sum = sum + term_n

        if verbose:
            print >> sys.stderr, '\tTerm: %d' % term,
            print >> sys.stderr, '\tterm_n: %e' % term_n,
            print >> sys.stderr, '\tsum: %e' % sum

        if factor1 == zero:         # all further terms will be zero
            break
        #if log(term_n, '10') < -1*mpf.dps:
        if abs(term_n) < eps:
            break

        term = term + 1

    v2 = 2*q**(mpf('0.25'))*sum

    # calculate v3(0, q)
    sum = zero
    term = one                          # series starts at one
    while True:
        factor1 = q**(term*term)
        term_n = factor1                # suboptimal, kept for readability
        sum = sum + term_n

        if factor1 == mpf('0'):      # all further terms will be zero
            break
        #if log(term_n, '10') < -1*mpf.dps:
        if abs(term_n) < eps:
            break

        term = term + 1

    v3 = one + 2*sum

    k = v2**2/v3**2

    return k

def jacobi_theta_1(z, m, verbose=False):
    """
    Implements the jacobi theta function 1, using the series expansion
    found in Abramowitz & Stegun [1].

    z is any complex number, but only reals here?
    m is the parameter, which must be converted to the nome
    """
    if verbose:
        print >> sys.stderr, 'elliptic.jacobi_theta_1'

    m = convert_lossless(m)
    z = convert_lossless(z)

    k = sqrt(m)
    q = calculate_nome(k)

    if verbose:
        print >> sys.stderr, '\tk: %f ' % k
        print >> sys.stderr, '\tq: %f ' % q

    if abs(q) >= mpf('1'):
        raise ValueError

    sum = 0
    term = 0                                    # series starts at 0

    minusone = mpf('-1')
    zero = mpf('0')
    one = mpf('1')

    if z == zero:
        if verbose:
            print >> sys.stderr, 'elliptic.jacobi_theta_1: z == 0, return 0'
        return zero
    elif q == zero:
        if verbose:
            print >> sys.stderr, 'elliptic.jacobi_theta_1: q == 0, return 0'
        return zero
    else:
        if verbose:
            print >> sys.stderr, 'ellipticl.jacobi_theta_1: calculating'
        while True:
            if (term % 2) == 0:
                factor0 = one
            else:
                factor0 = minusone

            factor1 = q**(term*(term +1))
            factor2 = sin((2*term + 1)*z)

            term_n = factor0 * factor1 * factor2
            sum = sum + term_n
            if verbose:
                print >> sys.stderr, '\tTerm: %d' % term,
                print >> sys.stderr, '\tterm_n: %e' % term_n,
                print >> sys.stderr, '\tsum: %e' % sum

            if factor1 == zero:         # all further terms will be zero
                break
            if factor2 != zero:         # check precision iff cos != 0
                #if log(term_n, '10') < -1*mpf.dps:
                if abs(term_n) < eps:
                    break

            term = term + 1

        return (2*q**(0.25))*sum

    # can't get here
    print >> sys.stderr, 'elliptic.jacobi_theta_1 in impossible state'
    sys.exit(2)

def jacobi_theta_2(z, m, verbose=False):
    """
    Implements the jacobi theta function 2, using the series expansion
    found in Abramowitz & Stegun [4].

    z is any complex number, but only reals here?
    m is the parameter, which must be converted to the nome
    """
    if verbose:
        print >> sys.stderr, 'elliptic.jacobi_theta_2'

    m = convert_lossless(m)
    z = convert_lossless(z)

    k = sqrt(m)
    q = calculate_nome(k)

    if verbose:
        print >> sys.stderr, '\tk: %f ' % k
        print >> sys.stderr, '\tq: %f ' % q

    if abs(q) >= mpf('1'):
        raise ValueError

    sum = 0
    term = 0                    # series starts at zero

    zero = mpf('0')

    if q == zero:
        if verbose:
            print >> sys.stderr, 'elliptic.jacobi_theta_2: q == 0, return 0'
        return zero
    else:
        if verbose:
            print >> sys.stderr, 'elliptic.jacobi_theta_2: calculating'
        if z == zero:
            factor2 = mpf('1')
            while True:
                factor1 = q**(term*(term + 1))
                term_n = factor1            # suboptimal, kept for readability
                sum = sum + term_n

                if verbose:
                    print >> sys.stderr, '\tTerm: %d' % term,
                    print >> sys.stderr, '\tterm_n: %e' % term_n,
                    print >> sys.stderr, '\tsum: %e' % sum

                if factor1 == zero:         # all further terms will be zero
                    break
                #if log(term_n, '10') < -1*mpf.dps:
                if abs(term_n) < eps:
                    break

                term = term + 1
        else:
            while True:
                factor1 = q**(term*(term + 1))
                if z == zero:
                    factor2 = 1
                else:
                    factor2 = cos((2*term + 1)*z)

                term_n = factor1 * factor2
                sum = sum + term_n

                if verbose:
                    print >> sys.stderr, '\tTerm: %d' % term,
                    print >> sys.stderr, '\tterm_n: %e' % term_n,
                    print >> sys.stderr, '\tsum: %e' % sum

                if factor1 == zero:         # all further terms will be zero
                    break
                if factor2 != zero:         # check precision iff cos != 0
                    #if log(term_n, '10') < -1*mpf.dps:
                    if abs(term_n) < eps:
                        break

                term = term + 1

        return (2*q**(0.25))*sum

    # can't get here
    print >> sys.stderr, 'elliptic.jacobi_theta_2 in impossible state'
    sys.exit(2)

def jacobi_theta_3(z, m):
    """
    Implements the jacobi theta function 2, using the series expansion
    found in Abramowitz & Stegun [4].

    z is any complex number, but only reals here?
    m is the parameter, which must be converted to the nome
    """
    m = convert_lossless(m)
    z = convert_lossless(z)

    k = sqrt(m)
    q = calculate_nome(k)

    if abs(q) >= mpf('1'):
        raise ValueError

    delta_sum = 1
    term = 1                    # series starts at 1

    zero = mpf('0')
    sum = zero

    if z == zero:
        factor2 = mpf('1')
        while True:

            factor1 = q**(term*term)
            term_n = factor1                # suboptimal, kept for readability
            sum = sum + term_n

            if factor1 == mpf('0'):      # all further terms will be zero
                break
            #if log(term_n, '10') < -1*mpf.dps:
            if abs(term_n) < eps:
                break

            term = term + 1

    else:
        while True:
            factor1 = q**(term*term)
            if z == zero:
                factor2 = 1
            else:
                factor2 = cos(2*term*z)

            term_n = factor1 * factor2
            sum = sum + term_n

            if factor1 == mpf('0'):      # all further terms will be zero
                break
            if factor2 != mpf('0'):      # check precision iff cos != 0
                #if log(term_n, '10') < -1*mpf.dps:
                if abs(term_n) < eps:
                    break

            term = term + 1

    return 1 + 2*sum

def jacobi_theta_4(z, m):
    """
    Implements the series expansion of the jacobi theta function
    1, where z == 0.

    z is any complex number, but only reals here?
    m is the parameter, which must be converted to the nome
    """
    m = convert_lossless(m)
    z = convert_lossless(z)

    k = sqrt(m)
    q = calculate_nome(k)

    if abs(q) >= mpf('1'):
        raise ValueError

    sum = 0
    delta_sum = 1
    term = 1                    # series starts at 1

    zero = mpf('0')

    if z == zero:
        factor2 = mpf('1')
        while True:
            if (term % 2) == 0:
                factor0 = 1
            else:
                factor0 = -1
            factor1 = q**(term*term)
            term_n = factor0 * factor1 * factor2
            sum = sum + term_n

            if factor1 == mpf('0'):      # all further terms will be zero
                break
            #if log(term_n, '10') < -1*mpf.dps:
            if abs(term_n) < eps:
                break

            term = term + 1
    else:
        while True:
            if (term % 2) == 0:
                factor0 = 1
            else:
                factor0 = -1
            factor1 = q**(term*term)
            if z == zero:
                factor2 = 1
            else:
                factor2 = cos(2*term*z)

            term_n = factor0 * factor1 * factor2
            sum = sum + term_n

            if factor1 == mpf('0'):      # all further terms will be zero
                break
            if factor2 != mpf('0'):      # check precision iff cos != 0
                #if log(term_n, '10') < -1*mpf.dps:
                if abs(term_n) < eps:
                    break

            term = term + 1

    return 1 + 2*sum

def jacobi_elliptic_sn(u, m, verbose=False):
    """
    Implements the jacobi elliptic sn function, using the expansion in
    terms of q, from Abramowitz 16.23.1.

    u is any complex number, m is the parameter

    Alternative implementation:

    Expansion in terms of jacobi theta functions appears to fail with
    round off error, despite   I also think that the expansion in
    terms of q is much faster than four expansions in terms of q.

    **********************************
    Previous implementation kept here:

    if not isinstance(u, mpf):
        raise TypeError
    if not isinstance(m, mpf):
        raise TypeError

    zero = mpf('0')

    if u == zero and m == 0:
        return zero
    else:
        q = calculate_nome(sqrt(m))

        v3 = jacobi_theta_3(zero, q)
        v2 = jacobi_theta_2(zero, q)        # mathworld says v4

        arg1 = u / (v3*v3)

        v1 = jacobi_theta_1(arg1, q)
        v4 = jacobi_theta_4(arg1, q)

        sn = (v3/v2)*(v1/v4)

        return sn
    **********************************
    """
    u = convert_lossless(u)
    m = convert_lossless(m)

    if verbose:
        print >> sys.stderr, '\nelliptic.jacobi_elliptic_sn'
        print >> sys.stderr, '\tu: %1.12f' % u
        print >> sys.stderr, '\tm: %1.12f' % m

    zero = mpf('0')
    onehalf = mpf('0.5')
    one = mpf('1')
    two = mpf('2')

    if m == zero:           # sn collapes to sin(u)
        if verbose:
            print >> sys.stderr, '\nsn: special case, m == 0'
        return sin(u)
    elif m == one:          # sn collapses to tanh(u)
        if verbose:
            print >> sys.stderr, '\nsn: special case, m == 1'
        return tanh(u)
    else:
        k = sqrt(m)                        # convert m to k
        q = calculate_nome(k)
        v = (pi * u) / (two*ellipk(k**2))

    if v == pi or v == zero:     # sin factor always zero
        return zero

    sum = zero
    term = zero                     # series starts at zero

    while True:
        if verbose:
            print >> sys.stderr, 'elliptic.jacobi_elliptic_sn: calculating'
        factor1 = (q**(term + onehalf)) / (one - q**(two*term + one))
        factor2 = sin((two*term + one)*v)

        term_n = factor1*factor2
        sum = sum + term_n

        if verbose:
            print >> sys.stderr, '\tTerm: %d' % term,
            print >> sys.stderr, '\tterm_n: %e' % term_n,
            print >> sys.stderr, '\tsum: %e' % sum

        if not factor2 == zero:
            #if log(term_n, '10') < -1*mpf.dps:
            if abs(term_n) < eps:
                break

        term = term + one

    answer = (two*pi) / (sqrt(m) * ellipk(k**2)) * sum

    return answer

def jacobi_elliptic_cn(u, m, verbose=False):
    """
    Implements the jacobi elliptic cn function, using the expansion in
    terms of q, from Abramowitz 16.23.2.
    """
    u = convert_lossless(u)
    m = convert_lossless(m)

    if verbose:
        print >> sys.stderr, '\nelliptic.jacobi_elliptic_cn'
        print >> sys.stderr, '\tu: %1.12f' % u
        print >> sys.stderr, '\tm: %1.12f' % m

    zero = mpf('0')
    onehalf = mpf('0.5')
    one = mpf('1')
    two = mpf('2')

    if m == zero:                   # cn collapses to cos(u)
        if verbose:
            print >> sys.stderr, 'cn: special case, m == 0'
        return cos(u)
    elif m == one:                  # cn collapses to sech(u)
        if verbose:
            print >> sys.stderr, 'cn: special case, m == 1'
        return sech(u)
    else:
        k = sqrt(m)                        # convert m to k
        q = calculate_nome(k)
        kprimesquared = one - k**2
        kprime = sqrt(kprimesquared)
        v = (pi * u) / (two*ellipk(k**2))

    sum = zero
    term = zero                     # series starts at zero

    if verbose:
        print >> sys.stderr, 'elliptic.jacobi_elliptic_cn: calculating'
    while True:
        factor1 = (q**(term + onehalf)) / (one + q**(two*term + one))
        factor2 = cos((two*term + one)*v)

        term_n = factor1*factor2
        sum = sum + term_n

        if verbose:
            print >> sys.stderr, '\tTerm: %d' % term,
            print >> sys.stderr, '\tterm_n: %e' % term_n,
            print >> sys.stderr, '\tsum: %e' % sum

        if not factor2 == zero:
            #if log(term_n, '10') < -1*mpf.dps:
            if abs(term_n) < eps:
                break

        term = term + one

    answer = (two*pi) / (sqrt(m) * ellipk(k**2)) * sum

    return answer

def jacobi_elliptic_dn(u, m, verbose=False):
    """
    Implements the jacobi elliptic cn function, using the expansion in
    terms of q, from Abramowitz 16.23.3.
    """
    u = convert_lossless(u)
    m = convert_lossless(m)

    if verbose:
        print >> sys.stderr, '\nelliptic.jacobi_elliptic_dn'
        print >> sys.stderr, '\tu: %1.12f' % u
        print >> sys.stderr, '\tm: %1.12f' % m

    zero = mpf('0')
    onehalf = mpf('0.5')
    one = mpf('1')
    two = mpf('2')

    if m == zero:           # dn collapes to 1
        return one
    elif m == one:          # dn collapses to sech(u)
        return sech(u)
    else:
        k = sqrt(m)                        # convert m to k
        q = calculate_nome(k)
        v = (pi * u) / (two*ellipk(k**2))

    sum = zero
    term = one                  # series starts at one

    if verbose:
        print >> sys.stderr, 'elliptic.jacobi_elliptic_dn: calculating'
    while True:
        factor1 = (q**term) / (one + q**(two*term))
        factor2 = cos(two*term*v)

        term_n = factor1*factor2
        sum = sum + term_n

        if verbose:
            print >> sys.stderr, '\tTerm: %d' % term,
            print >> sys.stderr, '\tterm_n: %e' % term_n,
            print >> sys.stderr, '\tsum: %e' % sum

        if not factor2 == zero:
            #if log(term_n, '10') < -1*mpf.dps:
            if abs(term_n) < eps:
                break

        term = term + one

    K = ellipk(k**2)
    answer = (pi / (two*K)) + (two*pi*sum)/(ellipk(k**2))

    return answer



#!/usr/bin/env python
import iam_sympy_example

"""
This example calculates the Ricci tensor from the metric and does this
on the example of Schwarzschild solution.
"""

from sympy import exp, Symbol, sin, Rational, Derivative, dsolve, Function, \
                  Matrix, Eq, pprint, Pow

def grad(f,X):
    a=[]
    for x in X:
        a.append( f.diff(x) )
    return a

def d(m,x):
    return grad(m[0,0],x)

class MT(object):
    def __init__(self,m):
        self.gdd=m
        self.guu=m.inv()

    def __str__(self):
        return "g_dd =\n" + str(self.gdd)

    def dd(self,i,j):
        return self.gdd[i,j]

    def uu(self,i,j):
        return self.guu[i,j]

class G(object):
    def __init__(self,g,x):
        self.g = g
        self.x = x

    def udd(self,i,k,l):
        g=self.g
        x=self.x
        r=0
        for m in [0,1,2,3]:
            r+=g.uu(i,m)/2 * (g.dd(m,k).diff(x[l])+g.dd(m,l).diff(x[k]) \
                    - g.dd(k,l).diff(x[m]))
        return r

class Riemann(object):
    def __init__(self,G,x):
        self.G = G
        self.x = x

    def uddd(self,rho,sigma,mu,nu):
        G=self.G
        x=self.x
        r=G.udd(rho,nu,sigma).diff(x[mu])-G.udd(rho,mu,sigma).diff(x[nu])
        for lam in [0,1,2,3]:
            r+=G.udd(rho,mu,lam)*G.udd(lam,nu,sigma) \
                -G.udd(rho,nu,lam)*G.udd(lam,mu,sigma)
        return r

class Ricci(object):
    def __init__(self,R,x):
        self.R = R
        self.x = x
        self.g = R.G.g

    def dd(self,mu,nu):
        R=self.R
        x=self.x
        r=0
        for lam in [0,1,2,3]:
            r+=R.uddd(lam,mu,lam,nu)
        return r

    def ud(self,mu,nu):
        r=0
        for lam in [0,1,2,3]:
            r+=self.g.uu(mu,lam)*self.dd(lam,nu)
        return r.expand()

def curvature(Rmn):
    return Rmn.ud(0,0)+Rmn.ud(1,1)+Rmn.ud(2,2)+Rmn.ud(3,3)

#class nu(Function):
#    def getname(self):
#        return r"\nu"
#        return r"nu"

#class lam(Function):
#    def getname(self):
#        return r"\lambda"
#        return r"lambda"
nu = Function("nu")
lam = Function("lamda")

t=Symbol("t")
r=Symbol("r")
theta=Symbol(r"theta")
phi=Symbol(r"phi")

#general, spherically symmetric metric
gdd=Matrix((
    (-exp(nu(r)),0,0,0),
    (0, exp(lam(r)), 0, 0),
    (0, 0, r**2, 0),
    (0, 0, 0, r**2*sin(theta)**2)
    ))
#spherical - flat
#gdd=Matrix((
#    (-1, 0, 0, 0),
#    (0, 1, 0, 0),
#    (0, 0, r**2, 0),
#    (0, 0, 0, r**2*sin(theta)**2)
#    ))
#polar - flat
#gdd=Matrix((
#    (-1, 0, 0, 0),
#    (0, 1, 0, 0),
#    (0, 0, 1, 0),
#    (0, 0, 0, r**2)
#    ))
#polar - on the sphere, on the north pole
#gdd=Matrix((
#    (-1, 0, 0, 0),
#    (0, 1, 0, 0),
#    (0, 0, r**2*sin(theta)**2, 0),
#    (0, 0, 0, r**2)
#    ))
g=MT(gdd)
X=(t,r,theta,phi)
Gamma=G(g,X)
Rmn=Ricci(Riemann(Gamma,X),X)

def pprint_Gamma_udd(i,k,l):
    pprint( Eq(Symbol('Gamma^%i_%i%i' % (i,k,l)), Gamma.udd(i,k,l))    )

def pprint_Rmn_dd(i,j):
    pprint( Eq(Symbol('R_%i%i' % (i,j)), Rmn.dd(i,j))    )

def main():

    print "Initial metric:"
    pprint(gdd)
    print "-"*40
    print "Christoffel symbols:"
    pprint_Gamma_udd(0,1,0)
    pprint_Gamma_udd(0,0,1)
    print
    pprint_Gamma_udd(1,0,0)
    pprint_Gamma_udd(1,1,1)
    pprint_Gamma_udd(1,2,2)
    pprint_Gamma_udd(1,3,3)
    print
    pprint_Gamma_udd(2,2,1)
    pprint_Gamma_udd(2,1,2)
    pprint_Gamma_udd(2,3,3)
    print
    pprint_Gamma_udd(3,2,3)
    pprint_Gamma_udd(3,3,2)
    pprint_Gamma_udd(3,1,3)
    pprint_Gamma_udd(3,3,1)
    print"-"*40
    print"Ricci tensor:"
    pprint_Rmn_dd(0,0)
    e =  Rmn.dd(1,1)
    pprint_Rmn_dd(1,1)
    pprint_Rmn_dd(2,2)
    pprint_Rmn_dd(3,3)
    #print
    #print "scalar curvature:"
    #print curvature(Rmn)
    print "-"*40
    print "solve the Einstein's equations:"
    e = e.subs(nu(r), -lam(r))
    l =  dsolve(e, [lam(r)])
    pprint( Eq(lam(r), l) )
    metric = gdd.subs(lam(r), l).subs(nu(r),-l)#.combine()
    print "metric:"
    pprint( metric )

if __name__ == "__main__":
    main()



from mptypes import (mp, mpf, convert_lossless, inf,
   eps, nstr, make_mpf, AS_POINTS)

from functions import pi, exp, log, ldexp

from libmpf import mpf_neg

import math

def NEG(x):
    return make_mpf(mpf_neg(x._mpf_))

def transform(f, a, b):
    """
    Given an integrand f defined over the interval [a, b], return an
    equivalent integrand g defined on the standard interval [-1, 1].

    If a and b are finite, this is achived by means of a linear change
    of variables. If at least one point is infinite, the substitution
    t = 1/x is used.
    """
    a = convert_lossless(a)
    b = convert_lossless(b)
    if (a, b) == (-1, 1):
        return f
    one = mpf(1)
    half = mpf(0.5)
    # The transformation 1/x sends [1, inf] to [0, 1], which in turn
    # can be transformed to [-1, 1] the usual way. For a double
    # infinite interval, we simply evaluate the function symmetrically
    if (a, b) == (-inf, inf):
        # return transform(lambda x: (f(-1/x+1)+f(1/x-1))/x**2, 0, 1)
        def y(x):
            u = 2/(x+one)
            w = one - u
            return half * (f(w)+f(-w)) * u**2
        return y
    if a == -inf:
        # return transform(lambda x: f(-1/x+b+1)/x**2, 0, 1)
        b1 = b+1
        def y(x):
            u = 2/(x+one)
            return half * f(b1-u) * u**2
        return y
    if b == inf:
        # return transform(lambda x: f(1/x+a-1)/x**2, 0, 1)
        a1 = a-1
        def y(x):
            u = 2/(x+one)
            return half * f(a1+u) * u**2
        return y
    # Simple linear change of variables
    C = (b-a)/2
    D = (b+a)/2
    def g(x):
        return C * f(D + C*x)
    return g


class Quadrature:

    @classmethod
    def get_nodes(cls, prec, level, verbose=False):
        if (prec, level) in cls.cached_nodes:
            return cls.cached_nodes[prec, level]
        orig = mp.prec
        try:
            nodes = cls.calc_nodes(prec, level, verbose)
        finally:
            mp.prec = orig
        cls.cached_nodes[prec, level] = nodes
        return nodes

    @classmethod
    def guess_level(cls, prec):
        """
        Estimate a reasonable default maximum level for quadrature that
        doubles the accuracy when the level is incremented.
        (This is the case for tanh-sinh quadrature as implemented here.)

            50 bits -> 6
            100 bits -> 7
            500 bits -> 10
            3000 bits -> 12

        These numbers are based purely on a limited amount of
        experimentation and will sometimes be wrong.
        """
        # Expected level
        g = int(4 + max(0, log(prec/30.0, 2)))
        # Reasonable "worst case"
        g += 2
        return g

    @classmethod
    def estimate_error(cls, results, prec, epsilon):
        """
        Estimate error of the calculation at the present level by
        comparing it to the results from two previous levels. The
        algorithm is given by Borwein, Bailey & Girgensohn.
        """
        try:
            if results[-1] == results[-2] == results[-3]:
                return mpf(0)
            D1 = log(abs(results[-1]-results[-2]), 10)
            D2 = log(abs(results[-1]-results[-3]), 10)
        except ValueError:
            return epsilon
        D3 = -prec
        D4 = min(0, max(D1**2/D2, 2*D1, D3))
        return mpf(10) ** int(D4)

    @classmethod
    def summation(cls, f, points, prec, epsilon, max_level, verbose=False):
        """
        Main summation function
        """
        I = err = mpf(0)
        for i in xrange(len(points)-1):
            a, b = points[i], points[i+1]
            if a == b:
                continue
            g = transform(f, a, b)
            results = []
            for level in xrange(1, max_level+1):
                if verbose:
                    print "Integrating from %s to %s (level %s of %s)" % \
                        (nstr(a), nstr(b), level, max_level)
                results.append(cls.sum_next(prec, level, results, g, verbose))
                if level > 2:
                    err = cls.estimate_error(results, prec, epsilon)
                    if err <= epsilon:
                        break
                    if verbose:
                        print "Estimated error:", nstr(err)
            I += results[-1]
        if err > epsilon:
            if verbose:
                print "Failed to reach full accuracy. Estimated error:", nstr(err)
        return I, err

class TanhSinh(Quadrature):
    """
    This class implements "tanh-sinh" or "doubly exponential"
    quadrature. This quadrature rule is based on the Euler-Maclaurin
    integral formula. By performing a change of variables involving
    nested exponentials / hyperbolic functions (hence the name), the
    derivatives at the endpoints vanish rapidly.

    Since the error term in the Euler-Maclaurin formula depends
    on the derivatives at the endpoints, a simple step sum becomes
    extremely accurate. In practice, this means that doubling the
    number of evaluation points roughly doubles the number of accurate
    digits.

    Comparison to Gauss-Legendre:
      * Initial computation of nodes is usually faster
      * Handles endpoint singularities better
      * Handles infinite integration intervals better
      * Is slower for smooth integrands once nodes have been computed

    The implementation of the tanh-sinh algorithm is based on the
    description given in Borwein, Bailey & Girgensohn, "Experimentation
    in Mathematics - Computational Paths to Discovery", A K Peters,
    2003, pages 312-313.

    A few improvements have been made:
      * A more efficient scheme is used to compute nodes
      * The nodes are computed successively instead of all at once

    Various documents describing the algorithm are available online, e.g.:

      * http://crd.lbl.gov/~dhbailey/dhbpapers/dhb-tanh-sinh.pdf
      * http://users.cs.dal.ca/~jborwein/tanh-sinh.pdf
    """

    cached_nodes = {}

    @classmethod
    def sum_next(cls, prec, level, previous, f, verbose=False):
        h = mpf(2)**(-level)
        # Abscissas overlap, so reusing saves half of the time
        if previous:
            S = previous[-1]/(h*2)
        else:
            S = mpf(0)
        for x, w in cls.get_nodes(prec, level, verbose=False):
            S += w*(f(NEG(x)) + f(x))
        return h*S

    @classmethod
    def calc_nodes(cls, prec, level, verbose=False):
        """
        The abscissas and weights for tanh-sinh quadrature are given by

            x[k] = tanh(pi/2 * sinh(t))
            w[k] = pi/2 * cosh(t) / cosh(pi/2 sinh(t))**2

        Here t varies uniformly with k: t0, t0+h, t0+2*h, ...

        The list of nodes is actually infinite, but the weights
        die off so rapidly that only a few are needed.
        """
        nodes = []

        extra = 20
        mp.prec += extra
        eps = ldexp(1, -prec-10)
        pi4 = pi/4

        # For simplicity, we work in steps h = 1/2^n, with the first point
        # offset so that we can reuse the sum from the previous level

        # We define level 1 to include the "level 0" steps, including
        # the point x = 0. (It doesn't work well otherwise; not sure why.)
        t0 = ldexp(1, -level)
        if level == 1:
            nodes.append((mpf(0), pi4))
            h = t0
        else:
            h = t0*2

        # Since h is fixed, we can compute the next exponential
        # by simply multiplying by exp(h)
        expt0 = exp(t0)
        a = pi4 * expt0
        b = pi4 / expt0
        udelta = exp(h)
        urdelta = 1/udelta

        for k in xrange(0, 20*2**level+1):
            # Reference implementation:
            # t = t0 + k*h
            # x = tanh(pi/2 * sinh(t))
            # w = pi/2 * cosh(t) / cosh(pi/2 * sinh(t))**2

            # Fast implementation. Note that c = exp(pi/2 * sinh(t))
            c = exp(a-b)
            d = 1/c
            co = (c+d)/2
            si = (c-d)/2
            x = si / co
            w = (a+b) / co**2
            diff = abs(x-1)
            if diff <= eps:
                break

            nodes.append((x, w))
            a *= udelta
            b *= urdelta

            if verbose and k % 300 == 150:
                # Note: the number displayed is rather arbitrary. Should
                # figure out how to print something that looks more like a
                # percentage
                print "Calculating nodes:", nstr(-log(diff, 10) / prec)

        mp.prec -= extra
        return nodes


class GaussLegendre(Quadrature):
    """
    This class implements Gauss-Legendre quadrature, which is
    exceptionally efficient for polynomials and polynomial-like (i.e.
    very smooth) integrands.

    The abscissas and weights are given by roots and values of
    Legendre polynomials, which are the orthogonal polynomials
    on [-1, 1] with respect to the unit weight.

    In this implementation, we take the "level" m of the quadrature
    to denote a Gauss-Legendre rule of degree 3*2^m (following Borwein,
    Bailey & Girgensohn). This ensures quadratic convergence.

    Comparison to tanh-sinh quadrature:
      * Is faster for smooth integrands once nodes have been computed
      * Initial computation of nodes is usually slower
      * Handles endpoint singularities worse
      * Handles infinite integration intervals worse

    """

    cached_nodes = {}

    @classmethod
    def calc_nodes(cls, prec, level, verbose=False):
        # It is important that the epsilon is set lower than the
        # "real" epsilon
        epsilon = ldexp(1, -prec-8)
        # Fairly high precision might be required for accurate
        # evaluation of the roots
        orig = mp.prec
        mp.prec = int(prec*1.5)
        nodes = []
        n = 3*2**(level-1)
        upto = n//2 + 1
        for j in xrange(1, upto):
            # Asymptotic formula for the roots
            r = mpf(math.cos(math.pi*(j-0.25)/(n+0.5)))
            # Newton iteration
            while 1:
                t1, t2 = 1, 0
                # Evaluates the Legendre polynomial using its defining
                # recurrence relation
                for j1 in xrange(1,n+1):
                    t3, t2, t1 = t2, t1, ((2*j1-1)*r*t1 - (j1-1)*t2)/j1
                t4 = n*(r*t1- t2)/(r**2-1)
                t5 = r
                a = t1/t4
                r = r - a
                if abs(a) < epsilon:
                    break
            x = r
            w = 2/((1-r**2)*t4**2)
            if verbose  and j % 30 == 15:
                print "Computing nodes (%i of %i)" % (j, upto)
            nodes.append((x, w))
        mp.prec = orig
        return nodes

    @classmethod
    def sum_next(cls, prec, level, previous, f, verbose=False):
        s = mpf(0)
        for x, w in cls.get_nodes(prec, level, verbose):
            s += w * (f(NEG(x)) + f(x))
        return s


def quad(f, *points, **kwargs):
    """
    Computes a single, double or triple integral over a given
    interval, rectangle, or box.

        quad(f(x), [x1, x2])
        quad(f(x,y), [x1, x2], [y1, y2])
        quad(f(x,y,z), [x1, x2], [y1, y2], [z1, z2])

    By default, tanh-sinh quadrature is used. A custom method
    can be specified via the 'method' keyword argument. Basic examples:

        >>> from mpmath import *
        >>> print quad(lambda x: exp(-x**2), [0, inf])
        0.886226925452758
        >>> f = lambda x, y: exp(x*sin(y))
        >>> print quad(f, [0, 1], [0, pi])
        4.47046663466179

    The functions quadgl(...) and quadts(...) act as shortcuts for
    quad(..., method='gauss-legendre') and quad(..., method='tanh-sinh').

    An interval may contain more than two points. In this case, the
    integration is split into subintervals, between each pair of
    consecutive points. This is useful for dealing with
    mid-interval discontinuities, or integrating over large
    intervals where the function is irregular or oscillates:

        >>> print quadgl(lambda x: abs(sin(x)), [0, pi, 2*pi])
        4.0
        >>> print quadgl(sin, arange(0, 1000+1, 10))
        0.437620923709297
        >>> print cos(0) - cos(1000)
        0.437620923709297

    Additional keyword options:

        verbose=True -- print details about progress

        error=True   -- return (value, err) where err is the estimated
                        error

    """
    rule = kwargs.get('method', TanhSinh)
    if type(rule) is str:
        rule = {'tanh-sinh':TanhSinh, 'gauss-legendre':GaussLegendre}[rule]
    verbose = kwargs.get('verbose')
    dim = len(points)
    orig = prec = mp.prec
    epsilon = eps/8
    m = kwargs.get('maxlevel') or rule.guess_level(prec)
    points = [AS_POINTS(p) for p in points]
    try:
        mp.prec += 20
        if dim == 1:
            v, err = rule.summation(f, points[0], prec, epsilon, m, verbose)
        elif dim == 2:
            v, err = rule.summation(lambda x: \
                    rule.summation(lambda y: f(x,y), \
                    points[1], prec, epsilon, m)[0],
                points[0], prec, epsilon, m, verbose)
        elif dim == 3:
            v, err = rule.summation(lambda x: \
                    rule.summation(lambda y: \
                        rule.summation(lambda z: f(x,y,z), \
                        points[2], prec, epsilon, m)[0],
                    points[1], prec, epsilon, m)[0],
                points[0], prec, epsilon, m, verbose)
        else:
            raise NotImplementedError("quadrature must have dim 1, 2 or 3")
    finally:
        mp.prec = orig
    if kwargs.get("error"):
        return +v, err
    return +v

def quadts(*args, **kwargs):
    """
    Performs tanh-sinh quadrature. The call

        quadts(func, *points, ...)

    is simply a shortcut for:

        quad(func, *points, ..., method=TanhSinh)

    For example, a single integral and a double integral:

        quadts(lambda x: exp(cos(x)), [0, 1])
        quadts(lambda x, y: exp(cos(x+y)), [0, 1], [0, 1])

    See the documentation for quad for information about how points
    arguments and keyword arguments are parsed.

    See documentation for TanhSinh for algorithmic information about
    tanh-sinh quadrature.
    """
    kwargs['method'] = TanhSinh
    return quad(*args, **kwargs)

def quadgl(*args, **kwargs):
    """
    Performs Gauss-Legendre quadrature. The call

        quadgl(func, *points, ...)

    is simply a shortcut for:

        quad(func, *points, ..., method=TanhSinh)

    For example, a single integral and a double integral:

        quadgl(lambda x: exp(cos(x)), [0, 1])
        quadgl(lambda x, y: exp(cos(x+y)), [0, 1], [0, 1])

    See the documentation for quad for information about how points
    arguments and keyword arguments are parsed.

    See documentation for TanhSinh for algorithmic information about
    tanh-sinh quadrature.
    """
    kwargs['method'] = GaussLegendre
    return quad(*args, **kwargs)




"""Univariate polynomials with galois field coefficients."""

import random

import modint
import sparse_poly

def GFPolyFactory(p):
    """Create custom class for specific coefficient type."""
    coefficient_type = modint.ModularIntegerFactory(p)
    class newClass(sparse_poly.SparsePolynomial):
        coeff_type = coefficient_type
        zero = coeff_type(0)

        @staticmethod
        def from_int_dict(int_dict):
            """Alternative construction, through integers."""
            result_dict = {}
            for e, c in int_dict.iteritems():
                cc = coefficient_type(c)
                if cc:
                    result_dict[e] = cc
            return newClass(result_dict)

        def to_int_dict(self):
            """Returns the dictionaries of integer representators."""
            result_dict = {}
            for e, c in self.coeffs.iteritems():
                result_dict[e] = c.value
            return result_dict

        def to_sym_int_dict(self):
            """Returns the dictionaries of symmetric integer representators."""
            result_dict = {}
            for e, c in self.coeffs.iteritems():
                result_dict[e] = int(c)
            return result_dict

        @staticmethod
        def random(min_degree, max_degree):
            """Generate random polynomial in given degree range."""
            degree = random.randrange(min_degree, max_degree+1)
            p = coefficient_type.modulus
            poly = {}

            while not poly:
                for monom in xrange(0, degree+1):
                    coeff = coefficient_type(random.randrange(p))

                    if coeff:
                        poly[monom] = coeff

            return newClass(poly)

        def monic(self):
            if not self:
                return self.coeff_type(0), self
            leading_coeff = self[self.degree]
            return leading_coeff, self.scale(self.coeff_type(1)/leading_coeff)

    newClass.__name__ = "%sPoly" % coefficient_type.__name__
    return newClass


# Division algorithms:

def div(f, g):
    """Division with remainder."""
    q = f.__class__()
    r = f
    if not g:
        return q, r
    deg_diff = r.degree - g.degree
    while deg_diff >= 0:
        quot = f.__class__({deg_diff: r[r.degree]/g[g.degree]})
        q += quot
        r -= quot*g
        deg_diff = r.degree - g.degree
    return q, r

def gcd(f, g):
    """Euclidean algorithm."""
    while g:
        f, g = g, div(f,g)[1]
    return f.monic()[1]

def lcm(f, g):
    q, r = div(f*g, gcd(f,g))
    assert not r
    return q.monic()[1]

def xgcd(f, g):
    """Extended euclidean algorithm.

    Outputs the gcd, s and t, such that:
        h == s*f + t*g

    """
    one = f.coeff_type(1)
    p, q, r, s, t  = [], [], [], [], []
    pp, rr = f.monic()
    p.append(pp)
    r.append(rr)
    pp, rr = g.monic()
    p.append(pp)
    r.append(rr)
    s.append(f.__class__({0:(one/p[0])}))
    s.append(f.__class__())
    t.append(f.__class__())
    t.append(f.__class__({0:(one/p[1])}))

    while True:
        q.append(div(r[-2], r[-1])[0])
        pp, rr = (r[-2] - q[-1]*r[-1]).monic()
        if not rr:
            return r[-1], s[-1], t[-1]
        p.append(pp)
        r.append(rr)
        pp = one/pp
        s.append((s[-2] - q[-1]*s[-1]).scale(pp))
        t.append((t[-2] - q[-1]*t[-1]).scale(pp))

# Arithmetic modular a polynomial p:

def truncate(f, n):
    """The remainder from division by x**n."""
    result_dict = {}
    for e, c in f.coeffs.iteritems():
        if e < n:
            result_dict[e] = c
    return f.__class__(result_dict)


def pow_mod(f, n, p):
    """Repeated squaring."""
    assert isinstance(n, (int, long)) and n >= 0
    if n == 0:
        return f.__class__({0: f.__class__.coeff_type(1)})
    binary_n = []
    while n:
        if n % 2:
            binary_n.insert(0, 1)
            n = (n - 1) / 2
        else:
            binary_n.insert(0, 0)
            n /= 2
    result = div(f, p)[1]
    for k in binary_n[1:]:
        result *= result
        result = div(result, p)[1]
        if k:
            result *= f
            result = div(result, p)[1]
    return result


# Factorization:

def distinct_degree_factor(f):
    """Return a list of divisors.

    Each polynomial has only factors of a specific degree.
    """

    result = []
    coeff_type = f.__class__.coeff_type
    p = coeff_type.modulus
    x_poly = f.__class__({1: coeff_type(1)})
    one_poly = f.__class__({0: coeff_type(1)})
    h = x_poly
    while f != one_poly:
        h = pow_mod(h, p, f) # h <- h**p mod f
        g = gcd(h - x_poly, f)
        f, r = div(f, g)
        assert not r
        result.append(g)
        # Early abort:
        if f.degree < 2*(g.degree + 1):
            result.append(f)
            break
    return result

def edf(f, n):
    """Cantor-Zassenhaus: Equal Degree Factorizaton """

    # TBD #################
    coeff_type = f.__class__.coeff_type
    one_poly = f.__class__({0: coeff_type(1)})
    modulus = coeff_type.modulus
    ################# END #

    factors = [f]

    if f.degree > n:
        m = f.degree/n

        while len(factors) < m:
            r = f.random(0, 2*n-1)

            if modulus == 2:
                raise NotImplementedError
            else:
                h = pow_mod(r, (modulus**n-1)//2, f)

            g = gcd(f, h-one_poly)

            if g != one_poly and g != f:
                factors = edf(g, n) + edf(div(f, g)[0], n)

    return factors

def factor(f):
    """Factorization of a univariate polynomial over a Galois field.

    Returns a list of the leading coefficient of f and the monic
    factors with their multiplicities.
    """
    p = f.__class__.coeff_type.modulus
    leading_coeff, f = f.monic()
    one_poly = f.__class__({0: f.__class__.coeff_type(1)})
    x_poly = f.__class__({1: f.__class__.coeff_type(1)})
    h = x_poly
    i = 0
    result = [leading_coeff]

    while f != one_poly:
        i += 1
        # One distinct-degree factorization step.
        h = pow_mod(h, p, f) # h <- h**p mod f
        g = gcd(h - x_poly, f)
        if g != one_poly:
            # Equal-degree factorization for degree i:
            g_factors = edf(g, i)
            # Now determine multiplicities of factors.
            for gg in g_factors:
                e = 0
                q, r = div(f, gg)
                while not r: # gg**e divides f
                    e += 1
                    f = q
                    q, r = div(f, gg)
                result.append((gg, e))
    return result

def factor_sqf(f):
    """Factorization of a univariate square-free polynomial over a Galois field.

    Returns a list of the leading coefficient and the monic factors of f.
    """

    one_poly = f.__class__({0: f.__class__.coeff_type(1)})
    leading_coeff, f = f.monic()
    result = [leading_coeff]
    for degree, divisor in enumerate(distinct_degree_factor(f)):
        if divisor != one_poly:
            result += edf(divisor, degree+1)

    return result




"""Algorithms for the computation of Groebner bases"""

from sympy import Basic, S, Symbol, symbols
from sympy.polynomials.base import Polynomial, term_lcm, term_mult, \
        term_div, term_is_mult, term_cmp, merge_var
from sympy.polynomials import div_

def groebner(f, var=None, order=None, reduced=True):
    """Computes a (reduced) Groebner base for a given list of polynomials.

    Usage:
    ======
        The input consists of a list of polynomials, either as SymPy
        expressions or instances of Polynomials. In the first case,
        you should also specify the variables and the monomial order
        with the arguments 'var' and 'order'. Only the first
        polynomial is checked for its type, the rest is assumed to
        match.

        By default, this algorithm returns the unique reduced Groebner
        base for the given ideal. By setting reduced=False, you can
        prevent the reduction steps.

    Examples:
    =========
        >>> x, y = symbols('xy')
        >>> G = groebner([x**2 + y**3, y**2-x], order='lex')
        >>> for g in G: print g
        x - y**2
        y**3 + y**4

    Notes:
    ======
        Groebner bases are used to choose specific generators for a
        polynomial ideal. Because these bases are unique, you can
        check for ideal equality, by comparing the Groebner bases. To
        see if one polynomial lies in on ideal, divide by the elements
        in the base and see if the remainder if 0. They can also be
        applied to equation systems: By choosing lexicographic
        ordering, you can eliminate one variable at a time, given that
        the ideal is zero-dimensional (finite number of solutions).

        Here, an improved version of Buchberger's algorithm is
        used. For all pairs of polynomials, the s-polynomial is
        computed, by mutually eliminating the leading terms of these 2
        polynomials. It's remainder (after division by the base) is
        then added. Sometimes, it is easy to see, that one
        s-polynomial will be reduced to 0 before computing it. At the
        end, the base is reduced, by trying to eliminate as many terms
        as possible with the leading terms of other base elements. The
        final step is to make all polynomials monic.

    References:
    ===========
        Cox, Little, O'Shea: Ideals, Varieties and Algorithms,
        Springer, 2. edition, p. 87

    """

    if isinstance(f, Basic):
        f = [f]
    if not isinstance(f[0], Polynomial):
        if var is None:
            var = merge_var(*map(lambda p: p.atoms(Symbol), f))
        if isinstance(var, Symbol):
            var = [var]
        f = map(lambda p: Polynomial(p, var=var, order=order), f)

    # Filter out the zero elements.
    f = filter(lambda p: p.sympy_expr is not S.Zero, f)

    # Empty Ideal.
    if len(f) == 0:
        return [Polynomial(S.Zero)]

    # Stores the unchecked combinations for s-poly's.
    b = []
    s = len(f)
    for i in range(0, s-1):
        for j in range(i+1, s):
            b.append((i, j))

    while b:
        # TODO: Choose better (non-arbitrary) pair: sugar method?
        i, j = b[0]
        crit = False
        lcm = term_lcm(f[i].coeffs[0], f[j].coeffs[0])
        # Check if leading terms are relativly prime.
        if  lcm[1:] != term_mult(f[i].coeffs[0],
                                 f[j].coeffs[0])[1:]:
            # TODO: Don't operate on the whole lists, do nested ifs instead?
            kk = filter(lambda k: k!=i and k!=j,range(0, s))
            kk = filter(lambda k: not (min(i,k),max(i,k)) in b, kk)
            kk = filter(lambda k: not (min(j,k),max(j,k)) in b, kk)
            # Check if the lcm is divisible by another base element.
            kk = filter(lambda k: term_is_mult(lcm,f[k].coeffs[0]), kk)
            crit = not bool(kk)
        if crit:
            factor_i = Polynomial(coeffs=(term_div(lcm, f[i].coeffs[0]),),
                                  var=f[0].var, order=f[0].order)
            factor_j = Polynomial(coeffs=(term_div(lcm, f[j].coeffs[0]),),
                                  var=f[0].var, order=f[0].order)
            s_poly = f[i]*factor_i - f[j]*factor_j
            s_poly = div_.div(s_poly, f)[-1] # reduce
            if s_poly.sympy_expr is not S.Zero:
                # we still have to add it to the base.
                s += 1
                f.append(s_poly)
                for t in range(0, s-1): # With a new element come
                    b.append((t, s-1))  # new combinationas to test.
        b = b[1:] # Checked one more.

    # We now have one possible Groebner base, probably too big.
    if not reduced:
        return f

    # We can get rid of all elements, where the leading term can be
    # reduced in the ideal of the remaining leading terms, that is,
    # can be divided by one of the other leading terms.
    blacklist = []
    for p in f:
        if filter(lambda x: term_is_mult(p.coeffs[0], x.coeffs[0]),
               filter(lambda x: not x in blacklist and x != p, f)):
            blacklist.append(p)
    for p in blacklist:
        f.remove(p)

    # We can now sort the basis elements according to their leading
    # term.
    f.sort(cmp=lambda a,b: term_cmp(a.coeffs[0], b.coeffs[0],
                                    a.order), reverse=True)

    # Divide all basis elements by their leading coefficient, to get a
    # leading 1.
    for i, p in enumerate(f):
        c = p.coeffs[0][0]
        f[i] = Polynomial(coeffs=tuple(map(lambda t:(t[0]/c,) + t[1:],
                                           p.coeffs)),
                          var=p.var, order=p.order)

    # We now have a minimal Groebner basis, which is still not unique.
    # The next step is to reduce all basis elements in respect to the
    # rest of the base (without touching the leading terms).
    # As the basis is already sorted, the rest gets smaller each time.
    for i,p in enumerate(f[0:-1]):
        pp = div_.div(p, f[i+1:])[-1]
        f[i] = pp

    return f



# -*- encoding: utf-8 -*-

from mptypes import convert_lossless, extraprec, eps
from calculus import diff, diffc
from functions import sqrt, sign

##############
# 1D-SOLVERS #
##############

class Secant:
    """
    1d-solver generating pairs of approximative root and error.

    Needs starting points x0 and x1 close to the root.
    x1 defaults to x0 + 0.25.

    Pro:
    * converges fast
    Contra:
    * converges slowly for multiple roots
    """
    maxsteps = 30

    def __init__(self, f, x0, **kwargs):
        if len(x0) == 1:
            self.x0 = x0[0]
            self.x1 = self.x0 + 0.25
        elif len(x0) == 2:
            self.x0 = x0[0]
            self.x1 = x0[1]
        else:
            raise ValueError('expected 1 or 2 starting points, got %i' * len(x0))
        self.f = f

    def __iter__(self):
        f = self.f
        x0 = self.x0
        x1 = self.x1
        f0 = f(x0)
        while True:
            f1 = f(x1)
            l = x1 - x0
            if not l:
                break
            s = (f1 - f0) / l
            if not s:
                break
            x0, x1 = x1, x1 - f1/s
            f0 = f1
            yield x1, abs(l)

class MNewton:
    """
    1d-solver generating pairs of approximative root and error.

    Needs starting point x0 close to the root.
    Uses modified Newton's method that converges fast regardless of the
    multiplicity of the root.

    Pro:
    * converges fast for multiple roots
    Contra:
    * needs first and second derivative of f
    * 3 function evaluations per iteration
    """
    maxsteps = 20

    def __init__(self, f, x0, **kwargs):
        if not len(x0) == 1:
            raise ValueError('expected 1 starting point, got %i' * len(x0))
        self.x0 = x0[0]
        self.f = f
        if not 'df' in kwargs:
            def df(x):
                return diff(f, x)
        else:
            df = kwargs['df']
        self.df = df
        if not 'd2f' in kwargs:
            def d2f(x):
                return diff(df, x)
        else:
            d2f = kwargs['df']
        self.d2f = d2f

    def __iter__(self):
        x = self.x0
        f = self.f
        df = self.df
        d2f = self.d2f
        while True:
            prevx = x
            fx = f(x)
            if fx == 0:
                break
            dfx = df(x)
            d2fx = d2f(x)
            # x = x - F(x)/F'(x) with F(x) = f(x)/f'(x)
            x -= fx / (dfx - fx * d2fx / dfx)
            error = abs(x - prevx)
            yield x, error

class Halley:
    """
    1d-solver generating pairs of approximative root and error.

    Needs a starting point x0 close to the root.
    Uses Halley's method with cubic convergance rate.

    Pro:
    * converges even faster the Newton's method
    * useful when computing with *many* digits
    Contra:
    * needs first and second derivative of f
    * 3 function evaluations per iteration
    * converges slowly for multiple roots
    """

    maxsteps = 20

    def __init__(self, f, x0, **kwargs):
        if not len(x0) == 1:
            raise ValueError('expected 1 starting point, got %i' * len(x0))
        self.x0 = x0[0]
        self.f = f
        if not 'df' in kwargs:
            def df(x):
                return diff(f, x)
        else:
            df = kwargs['df']
        self.df = df
        if not 'd2f' in kwargs:
            def d2f(x):
                return diff(df, x)
        else:
            d2f = kwargs['df']
        self.d2f = d2f

    def __iter__(self):
        x = self.x0
        f = self.f
        df = self.df
        d2f = self.d2f
        while True:
            prevx = x
            fx = f(x)
            dfx = df(x)
            d2fx = d2f(x)
            x -=  2*fx*dfx / (2*dfx**2 - fx*d2fx)
            error = abs(x - prevx)
            yield x, error

class Muller:
    """
    1d-solver generating pairs of approximative root and error.

    Needs starting points x0, x1 and x2 close to the root.
    x1 defaults to x0 + 0.25; x2 to x1 + 0.25.
    Uses Muller's method that converges towards complex roots.

    Pro:
    * converges fast (somewhat faster than secant)
    * can find complex roots
    Contra:
    * converges slowly for multiple roots
    * may have complex values for real starting points and real roots

    http://en.wikipedia.org/wiki/M%C3%BCller%27s_method
    """
    maxsteps = 30

    def __init__(self, f, x0, **kwargs):
        if len(x0) == 1:
            self.x0 = x0[0]
            self.x1 = self.x0 + 0.25
            self.x2 = self.x1 + 0.25
        elif len(x0) == 2:
            self.x0 = x0[0]
            self.x1 = x0[1]
            self.x2 = self.x1 + 0.25
        elif len(x0) == 3:
            self.x0 = x0[0]
            self.x1 = x0[1]
            self.x2 = x0[2]
        else:
            raise ValueError('expected 1, 2 or 3 starting points, got %i'
                             % len(x0))
        self.f = f
        self.verbose = kwargs['verbose']

    def __iter__(self):
        f = self.f
        x0 = self.x0
        x1 = self.x1
        x2 = self.x2
        fx0 = f(x0)
        fx1 = f(x1)
        fx2 = f(x2)
        while True:
            # TODO: maybe refactoring with function for divided differences
            # calculate divided diffferences
            fx2x1 = (fx1 - fx2) / (x1 - x2)
            fx2x0 = (fx0 - fx2) / (x0 - x2)
            fx1x0 = (fx0 - fx1) / (x0 - x1)
            w = fx2x1 + fx2x0 - fx1x0
            fx2x1x0 = (fx1x0 - fx2x1) / (x0 - x2)
            if w == 0 and fx2x1x0 == 0:
                if self.verbose:
                    print 'canceled with'
                    print 'x0 =', x0, ', x1 =', x1, 'and x2 =', x2
                break
            x0 = x1
            fx0 = fx1
            x1 = x2
            fx1 = fx2
            # denominator should be as large as possible => choose sign
            r = sqrt(w**2 - 4*fx2*fx2x1x0)
            if abs(w - r) > abs(w + r):
                r = -r
            x2 -= 2*fx2 / (w + r)
            fx2 = f(x2)
            error = abs(x2 - x1)
            yield x2, error

# TODO: consider raising a ValueError when there's no sign change in a and b
class Bisection:
    """
    1d-solver generating pairs of approximative root and error.

    Uses bisection method to find a root of f in [a, b].
    Might fail for multiple roots (needs sign change).

    Pro:
    * robust and reliable
    Contra:
    * converges slowly
    * needs sign change
    """
    maxsteps = 100

    def __init__(self, f, x0, **kwargs):
        if len(x0) != 2:
            raise ValueError('expected interval of 2 points, got %i' * len(x0))
        self.f = f
        self.a = x0[0]
        self.b = x0[1]

    def __iter__(self):
        f = self.f
        a = self.a
        b = self.b
        l = b - a
        while True:
            m = 0.5 * (a + b) # TODO: ldexp?
            if f(m) * f(b) < 0:
                a = m
            else:
                b = m
            l /= 2
            yield (a + b)/2, abs(l)

def _getm(method):
    """
    Return a function to calculate m for Illinois-like methods.
    """
    if method == 'illinois':
        def getm(fz, fb):
            return 0.5
    elif method == 'pegasus':
        def getm(fz, fb):
            return fb/(fb + fz)
    elif method == 'anderson':
        def getm(fz, fb):
            m = 1 - fz/fb
            if m > 0:
                return m
            else:
                return 0.5
    else:
        raise ValueError, "method '%s' not recognized" % method
    return getm

class Illinois:
    """
    1d-solver generating pairs of approximative root and error.

    Uses Illinois method or similair to find a root of f in [a, b].
    Might fail for multiple roots (needs sign change).
    Combines bisect with secant (improved regula falsi).

    The only difference between the methods is the scaling factor m, which is
    used to ensure convergence (you can choose one using the 'method' keyword):
    Illinois method ('illinois'):       m = 0.5
    Pegasus method ('pegasus'):         m = fb/(fb + fz)
    Anderson-Björk method ('anderson'): m = 1 - fz/fb if positive else 0.5

    Pro:
    * converges very fast
    Contra:
    * has problems with multiple roots
    * needs sign change
    """
    maxsteps = 30

    def __init__(self, f, x0, **kwargs):
        if len(x0) != 2:
            raise ValueError('expected interval of 2 points, got %i' * len(x0))
        self.a = x0[0]
        self.b = x0[1]
        self.f = f
        self.tol = kwargs['tol']
        self.verbose = kwargs['verbose']
        self.method = kwargs.get('method', 'illinois')
        self.getm = _getm(self.method)
        if self.verbose:
            print 'using %s method' % self.method

    def __iter__(self):
        method = self.method
        f = self.f
        a = self.a
        b = self.b
        fa = f(a)
        fb = f(b)
        m = None
        while True:
            l = b - a
            s = (fb - fa) / l
            z = a - fa/s
            fz = f(z)
            if abs(fz) < self.tol:
                # TODO: better condition (when f is very flat)
                if self.verbose:
                    print 'canceled with z =', z
                yield z, l
                break
            if fz * fb < 0: # root in [z, b]
                a = b
                fa = fb
                b = z
                fb = fz
            else: # root in [a, z]
                m = getm(fz, fb)
                b = z
                fb = fz
                fa = m*fa # scale down to ensure convergence
            if self.verbose and m and not method == 'illinois':
                print 'm:', m
            yield (a + b)/2, abs(l)

def Pegasus(*args, **kwargs):
    """
    1d-solver generating pairs of approximative root and error.

    Uses Pegasus method to find a root of f in [a, b].
    Wrapper for illinois to use method='pegasus'.
    """
    kwargs['method'] = 'pegasus'
    return Illinois(*args, **kwargs)

def Anderson(*args, **kwargs):
    u"""
    1d-solver generating pairs of approximative root and error.

    Uses Anderson-Björk method to find a root of f in [a, b].
    Wrapper for illinois to use method='pegasus'.
    """
    kwargs['method'] = 'anderson'
    return Illinois(*args, **kwargs)

# TODO: check whether it's possible to combine it with Illinois stuff
class Ridder:
    """
    1d-solver generating pairs of approximative root and error.

    Ridders' method to find a root of f in [a, b].
    Is told to perform as well as Brent's method while being simpler.

    Pro:
    * very fast
    * simpler than Brent's method
    Contra:
    * two function evaluations per step
    * has problems with multiple roots
    * needs sign change

    http://en.wikipedia.org/wiki/Ridders%27_method
    """
    maxsteps=30

    def __init__(self, f, x0, **kwargs):
        self.f = f
        if len(x0) != 2:
            raise ValueError('expected interval of 2 points, got %i' * len(x0))
        self.x1 = x0[0]
        self.x2 = x0[1]
        self.verbose = kwargs['verbose']
        self.tol = kwargs['tol']

    def __iter__(self):
        f = self.f
        x1 = self.x1
        fx1 = f(x1)
        x2 = self.x2
        fx2 = f(x2)
        while True:
            x3 = 0.5*(x1 + x2)
            fx3 = f(x3)
            x4 = x3 + (x3 - x1) * sign(fx1 - fx2) * fx3 / sqrt(fx3**2 - fx1*fx2)
            fx4 = f(x4)
            if abs(fx4) < self.tol:
                # TODO: better condition (when f is very flat)
                if self.verbose:
                    print 'canceled with f(x4) =', fx4
                yield x4, abs(x1 - x2)
                break
            if fx4 * fx2 < 0: # root in [x4, x2]
                x1 = x4
                fx1 = fx4
            else: # root in [x1, x4]
                x2 = x4
                fx2 = fx4
            error = abs(x1 - x2)
            yield (x1 + x2)/2, error

class ANewton:
    """
    EXPERIMENTAL 1d-solver generating pairs of approximative root and error.

    Uses Newton's method modified to use Steffensens method when convergence is
    slow. (I. e. for multiple roots.)
    """
    maxsteps = 20

    def __init__(self, f, x0, **kwargs):
        if not len(x0) == 1:
            raise ValueError('expected 1 starting point, got %i' * len(x0))
        self.x0 = x0[0]
        self.f = f
        if not 'df' in kwargs:
            def df(x):
                return diff(f, x)
        else:
            df = kwargs['df']
        self.df = df
        def phi(x):
            return x - f(x) / df(x)
        self.phi = phi
        self.verbose = kwargs['verbose']

    def __iter__(self):
        x0 = self.x0
        f = self.f
        df = self.df
        phi = self.phi
        error = 0
        counter = 0
        while True:
            prevx = x0
            try:
                x0 = phi(x0)
            except ZeroDivisionError:
                if self.verbose:
                    'ZeroDivisionError: canceled with x =', x0
                break
            preverror = error
            error = abs(prevx - x0)
            # TODO: decide not to use convergence acceleration
            if error and abs(error - preverror) / error < 1:
                if self.verbose:
                    print 'converging slowly'
                counter += 1
            if counter >= 3:
                # accelerate convergence
                phi = steffensen(phi)
                counter = 0
                if self.verbose:
                    print 'accelerating convergence'
            yield x0, error

# TODO: add Brent

#############
# UTILITIES #
#############

str2solver = {'secant':Secant,'mnewton':MNewton, 'halley':Halley,
              'muller':Muller, 'bisect':Bisection, 'illinois':Illinois,
              'pegasus':Pegasus, 'anderson':Anderson, 'ridder':Ridder,
              'anewton':ANewton}

@extraprec(20)
def findroot(f, x0, solver=Secant, tol=None, verbose=False, verify=True,
             force_type=convert_lossless, **kwargs):
    """
    Find a root of f using x0 as starting point or interval.

    If not abs(f(root)) < tol an exception is raised.

    Arguments:
    f : one dimensional function
    x0 : starting point, several starting points or interval (depends on solver)
    tol : the returned solution has an error smaller than this
    verbose : print additional information for each iteration if true
    verify : verify the solution and raise a ValueError if abs(f(x)) > tol
    force_type : use specified type constructor on starting points
    solver : a generator for f and x0 returning approximative solution and error
    maxsteps : after how many steps the solver will cancel
    df : first derivative of f (used by some solvers)
    d2f : second derivative of f (used by some solvers)

    solver has to be callable with (f, x0, **kwargs) and return an generator
    yielding pairs of approximative solution and estimated error.
    You can use the following string aliases:
    'secant', 'mnewton', 'halley', 'muller', 'illinois', 'pegasus', 'anderson',
    'ridder', 'anewton', 'bisect'
    See mpmath.optimization for their documentation.
    """
    # initialize arguments
    if not force_type:
        force_type = lambda x: x
    elif not tol and (force_type == float or force_type == complex):
        tol = 2**(-52) # TODO: consider a less strict value
    kwargs['verbose'] = verbose
    if 'd1f' in kwargs:
        kwargs['df'] = kwargs['d1f']
    if tol is None:
        tol = eps * 2**10
    kwargs['tol'] = tol
    if isinstance(x0, (list, tuple)):
        x0 = [force_type(x) for x in x0]
    else:
        x0 = [force_type(x0)]
    if isinstance(solver, str):
        try:
            solver = str2solver[solver]
        except KeyError:
            raise ValueError('could not recognize solver')
    # use solver
    iterations = solver(f, x0, **kwargs)
    if 'maxsteps' in kwargs:
        maxsteps = kwargs['maxsteps']
    else:
        maxsteps = iterations.maxsteps
    i = 0
    for x, error in iterations:
        if verbose:
            print 'x:', x
            print 'error:', error
        i += 1
        if error < tol or i >= maxsteps:
            break
    if verify and abs(f(x))**2 > tol: # TODO: better condition?
        raise ValueError('Could not find root within given tolerance. '
                         '(%g > %g)\n'
                         'Try another starting point or tweak arguments.'
                         % (abs(f(x)), tol))
    return x

def multiplicity(f, root, tol=eps, maxsteps=10, **kwargs):
    """
    Return the multiplicity of a given root of f.

    Internally, numerical derivatives are used. This is very inefficient for
    higher order derviatives. You can be specify the n-th derivative using the
    dnf keyword.
    """
    kwargs['d0f'] = f
    for i in xrange(maxsteps):
        dfstr = 'd' + str(i) + 'f'
        if dfstr in kwargs:
            df = kwargs[dfstr]
        else:
            df = lambda x: diffc(f, x, i)
        if not abs(df(root)) < tol:
            break
    return i

def steffensen(f):
    """
    linear convergent function -> quadratic convergent function

    Steffensen's method for quadratic convergence of a linear converging
    sequence.
    Don not use it for higher rates of convergence.
    It may even work for divergent sequences.

    Definition:
    F(x) = (x*f(f(x)) - f(x)**2) / (f(f(x)) - 2*f(x) + x)
    """
    def F(x):
        fx = f(x)
        ffx = f(fx)
        return (x*ffx - fx**2) / (ffx - 2*fx + x)
    return F



"""
This module defines the mpf, mpc classes, and standard functions for
operating with them.
"""
__docformat__ = 'plaintext'

from settings import (MP_BASE, MP_ONE, mp, prec_rounding, extraprec, extradps,
    workprec, workdps, int_types, repr_dps, round_floor, round_ceiling)

from libmpf import (\
    ComplexResult, to_pickable, from_pickable, normalize,
    from_int, from_float, from_str, to_int, to_float, to_str,
    from_rational, from_man_exp,
    fone, fzero, finf, fninf, fnan,
    mpf_abs, mpf_pos, mpf_neg, mpf_add, mpf_sub, mpf_mul, mpf_mul_int,
    mpf_div, mpf_rdiv_int, mpf_pow_int, mpf_mod,
    mpf_eq, mpf_cmp, mpf_lt, mpf_gt, mpf_le, mpf_ge,
    mpf_hash, mpf_rand
)

from libmpc import (\
    complex_to_str,
    mpc_abs, mpc_add, mpc_add_mpf, mpc_sub, mpc_sub_mpf, mpc_mul, mpc_mul_mpf,
    mpc_mul_int, mpc_div, mpc_div_mpf, mpc_pow, mpc_pow_mpf, mpc_pow_int
)

from libelefun import mpf_pow

from libmpi import (\
    mpi_mid, mpi_delta, mpi_str,
    mpi_abs, mpi_pos, mpi_neg, mpi_add, mpi_sub,
    mpi_mul, mpi_div, mpi_pow_int, mpi_pow
)

class mpnumeric(object):
    """Base class for mpf and mpc. Calling mpnumeric(x) returns an mpf
    if x can be converted to an mpf (if it is a float, int, mpf, ...),
    and an mpc if x is complex."""
    __slots__ = []
    def __new__(cls, val):
        # TODO: should maybe normalize here
        if isinstance(val, cls): return val
        if isinstance(val, complex): return mpc(val)
        return mpf(val)

def convert_lossless(x, strings=True):
    """Attempt to convert x to an mpf or mpc losslessly. If x is an
    mpf or mpc, return it unchanged. If x is an int, create an mpf with
    sufficient precision to represent it exactly. If x is a str, just
    convert it to an mpf with the current working precision (perhaps
    this should be done differently...)"""
    if isinstance(x, mpnumeric): return x
    if isinstance(x, int_types): return make_mpf(from_int(x))
    if isinstance(x, float): return make_mpf(from_float(x))
    if isinstance(x, complex): return mpc(x)
    if strings and isinstance(x, basestring): return make_mpf(from_str(x, *prec_rounding))
    if hasattr(x, '_mpf_'): return make_mpf(x._mpf_)
    if hasattr(x, '_mpc_'): return make_mpc(x._mpc_)
    if hasattr(x, '_mpmath_'): return convert_lossless(x._mpmath_(*prec_rounding))
    raise TypeError("cannot create mpf from " + repr(x))

def try_convert_mpf_value(x, prec, rounding):
    if isinstance(x, float): return from_float(x)
    if hasattr(x, '_mpf_'): return x._mpf_
    if hasattr(x, '_mpmath_'):
        t = convert_lossless(x._mpmath_(prec, rounding))
        if isinstance(t, mpf):
            return t._mpf_
    return NotImplemented

def mpf_convert_arg(x, prec, rounding):
    if isinstance(x, int_types): return from_int(x)
    if isinstance(x, float): return from_float(x)
    if isinstance(x, basestring): return from_str(x, prec, rounding)
    if isinstance(x, constant): return x.func(prec, rounding)
    if hasattr(x, '_mpf_'): return x._mpf_
    if hasattr(x, '_mpmath_'):
        t = convert_lossless(x._mpmath_(prec, rounding))
        if isinstance(t, mpf):
            return t._mpf_
    raise TypeError("cannot create mpf from " + repr(x))

def mpf_convert_rhs(x):
    if isinstance(x, int_types): return from_int(x)
    if isinstance(x, float): return from_float(x)
    if isinstance(x, complex_types): return mpc(x)
    if hasattr(x, '_mpf_'): return x._mpf_
    if hasattr(x, '_mpmath_'):
        t = convert_lossless(x._mpmath_(*prec_rounding))
        if isinstance(t, mpf):
            return t._mpf_
        return t
    return NotImplemented

def mpf_convert_lhs(x):
    x = mpf_convert_rhs(x)
    if type(x) is tuple:
        return make_mpf(x)
    return x

def mpc_convert_lhs(x):
    try:
        return convert_lossless(x)
    except TypeError:
        return NotImplemented

new = object.__new__

class mpf(mpnumeric):
    """
    An mpf instance holds a real-valued floating-point number. mpf:s
    work analogously to Python floats, but support arbitrary-precision
    arithmetic.
    """
    __slots__ = ['_mpf_']

    def __new__(cls, val=fzero, **kwargs):
        """A new mpf can be created from a Python float, an int, a
        or a decimal string representing a number in floating-point
        format."""
        prec, rounding = prec_rounding
        if kwargs:
            prec = kwargs.get('prec', prec)
            if 'dps' in kwargs:
                prec = dps_to_prec(kwargs['dps'])
            rounding = kwargs.get('rounding', rounding)
        if type(val) is cls:
            sign, man, exp, bc = val._mpf_
            if (not man) and exp:
                return val
            return make_mpf(normalize(sign, man, exp, bc, prec, rounding))
        elif type(val) is tuple:
            if len(val) == 2:
                return make_mpf(from_man_exp(val[0], val[1], prec, rounding))
            if len(val) == 4:
                sign, man, exp, bc = val
                return make_mpf(normalize(sign, MP_BASE(man), exp, bc, prec, rounding))
            raise ValueError
        else:
            return make_mpf(mpf_pos(mpf_convert_arg(val, prec, rounding), prec, rounding))

    man_exp = property(lambda self: self._mpf_[1:3])
    man = property(lambda self: self._mpf_[1])
    exp = property(lambda self: self._mpf_[2])
    bc = property(lambda self: self._mpf_[3])

    real = property(lambda self: self)
    imag = property(lambda self: zero)

    def __getstate__(self): return to_pickable(self._mpf_)
    def __setstate__(self, val): self._mpf_ = from_pickable(val)

    def __repr__(s): return "mpf('%s')" % to_str(s._mpf_, repr_dps(mp.prec))
    def __str__(s): return to_str(s._mpf_, mp.dps)
    def __hash__(s): return mpf_hash(s._mpf_)
    def __int__(s): return int(to_int(s._mpf_))
    def __long__(s): return long(to_int(s._mpf_))
    def __float__(s): return to_float(s._mpf_)
    def __complex__(s): return complex(float(s))
    def __nonzero__(s): return s._mpf_ != fzero
    def __abs__(s): return make_mpf(mpf_abs(s._mpf_, *prec_rounding))
    def __pos__(s): return make_mpf(mpf_pos(s._mpf_, *prec_rounding))
    def __neg__(s): return make_mpf(mpf_neg(s._mpf_, *prec_rounding))

    def _cmp(s, t, func):
        if hasattr(t, '_mpf_'):
            t = t._mpf_
        else:
            t = mpf_convert_rhs(t)
            if t is NotImplemented:
                return t
        return func(s._mpf_, t)

    def __cmp__(s, t): return s._cmp(t, mpf_cmp)
    def __lt__(s, t): return s._cmp(t, mpf_lt)
    def __gt__(s, t): return s._cmp(t, mpf_gt)
    def __le__(s, t): return s._cmp(t, mpf_le)
    def __ge__(s, t): return s._cmp(t, mpf_ge)

    def __ne__(s, t):
        v = s.__eq__(t)
        if v is NotImplemented:
            return v
        return not v

    def __rsub__(s, t):
        prec, rounding = prec_rounding
        if type(t) in int_types:
            return make_mpf(mpf_sub(from_int(t), s._mpf_, prec, rounding))
        t = mpf_convert_lhs(t)
        if t is NotImplemented:
            return t
        return t - s

    def __rdiv__(s, t):
        prec, rounding = prec_rounding
        if isinstance(t, int_types):
            return make_mpf(mpf_rdiv_int(t, s._mpf_, prec, rounding))
        t = mpf_convert_lhs(t)
        if t is NotImplemented:
            return t
        return t / s

    def __rpow__(s, t):
        t = mpf_convert_lhs(t)
        if t is NotImplemented:
            return t
        return t ** s

    def __rmod__(s, t):
        t = mpf_convert_lhs(t)
        if t is NotImplemented:
            return t
        return t % s

    def sqrt(s):
        from functions import sqrt
        return sqrt(s)

    def ae(s, t, rel_eps=None, abs_eps=None):
        return almosteq(s, t, rel_eps, abs_eps)


mpf_binary_op = """
def %NAME%(self, other):
    prec, rounding = prec_rounding
    sval = self._mpf_
    if hasattr(other, '_mpf_'):
    #try:
        tval = other._mpf_
        %WITH_MPF%
    #except AttributeError:
    #    pass
    ttype = type(other)
    if ttype in int_types:
        %WITH_INT%
    elif ttype is float:
        tval = from_float(other)
        %WITH_MPF%
    elif ttype is mpc:
        tval = other._mpc_
        %WITH_MPC%
    elif ttype is complex:
        tval = from_float(other.real), from_float(other.imag)
        %WITH_MPC%
    if isinstance(other, mpnumeric):
        return NotImplemented
    try:
        other = convert_lossless(other, strings=False)
    except TypeError:
        return NotImplemented
    return self.%NAME%(other)
"""

return_mpf = "; obj = new(mpf); obj._mpf_ = val; return obj"
return_mpc = "; obj = new(mpc); obj._mpc_ = val; return obj"

mpf_pow_same = """
        try:
            val = mpf_pow(sval, tval, prec, rounding) %s
        except ComplexResult:
            if mp.trap_complex:
                raise
            val = mpc_pow((sval, fzero), (tval, fzero), prec, rounding) %s
""" % (return_mpf, return_mpc)

def binary_op(name, with_mpf='', with_int='', with_mpc=''):
    code = mpf_binary_op
    code = code.replace("%WITH_INT%", with_int)
    code = code.replace("%WITH_MPC%", with_mpc)
    code = code.replace("%WITH_MPF%", with_mpf)
    code = code.replace("%NAME%", name)
    np = {}
    exec code in globals(), np
    return np[name]

mpf.__eq__ = binary_op('__eq__',
    'return mpf_eq(sval, tval)',
    'return mpf_eq(sval, from_int(other))',
    'return (tval[1] == fzero) and mpf_eq(tval[0], sval)')

mpf.__add__ = binary_op('__add__',
    'val = mpf_add(sval, tval, prec, rounding)' + return_mpf,
    'val = mpf_add(sval, from_int(other), prec, rounding)' + return_mpf,
    'val = mpc_add_mpf(tval, sval, prec, rounding)' + return_mpc)

mpf.__sub__ = binary_op('__sub__',
    'val = mpf_sub(sval, tval, prec, rounding)' + return_mpf,
    'val = mpf_sub(sval, from_int(other), prec, rounding)' + return_mpf,
    'val = mpc_sub((sval, fzero), tval, prec, rounding)' + return_mpc)

mpf.__mul__ = binary_op('__mul__',
    'val = mpf_mul(sval, tval, prec, rounding)' + return_mpf,
    'val = mpf_mul_int(sval, other, prec, rounding)' + return_mpf,
    'val = mpc_mul_mpf(tval, sval, prec, rounding)' + return_mpc)

mpf.__div__ = binary_op('__div__',
    'val = mpf_div(sval, tval, prec, rounding)' + return_mpf,
    'val = mpf_div(sval, from_int(other), prec, rounding)' + return_mpf,
    'val = mpc_div((sval, fzero), tval, prec, rounding)' + return_mpc)

mpf.__mod__ = binary_op('__mod__',
    'val = mpf_mod(sval, tval, prec, rounding)' + return_mpf,
    'val = mpf_mod(sval, from_int(other), prec, rounding)' + return_mpf,
    'raise NotImplementedError("complex modulo")')

mpf.__pow__ = binary_op('__pow__',
    mpf_pow_same,
    'val = mpf_pow_int(sval, other, prec, rounding)' + return_mpf,
    'val = mpc_pow((sval, fzero), tval, prec, rounding)' + return_mpc)

mpf.__radd__ = mpf.__add__
mpf.__rmul__ = mpf.__mul__
mpf.__truediv__ = mpf.__div__
mpf.__rtruediv__ = mpf.__rdiv__


class mpc(mpnumeric):
    """
    An mpc represents a complex number using a pair of mpf:s (one
    for the real part and another for the imaginary part.) The mpc
    class behaves fairly similarly to Python's complex type.
    """

    __slots__ = ['_mpc_']

    def __new__(cls, real=0, imag=0):
        s = object.__new__(cls)
        if isinstance(real, complex_types):
            real, imag = real.real, real.imag
        elif hasattr(real, "_mpc_"):
            s._mpc_ = real._mpc_
            return s
        real = mpf(real)
        imag = mpf(imag)
        s._mpc_ = (real._mpf_, imag._mpf_)
        return s

    real = property(lambda self: make_mpf(self._mpc_[0]))
    imag = property(lambda self: make_mpf(self._mpc_[1]))

    def __getstate__(self):
        return to_pickable(self._mpc_[0]), to_pickable(self._mpc_[1])

    def __setstate__(self, val):
        self._mpc_ = from_pickable(val[0]), from_pickable(val[1])

    def __repr__(s):
        r = repr(s.real)[4:-1]
        i = repr(s.imag)[4:-1]
        return "mpc(real=%s, imag=%s)" % (r, i)

    def __str__(s):
        return "(%s)" % complex_to_str(s.real._mpf_, s.imag._mpf_, mp.dps)

    def __complex__(s): return complex(float(s.real), float(s.imag))
    def __pos__(s): return mpc(s.real, s.imag)
    def __abs__(s): return make_mpf(mpc_abs(s._mpc_, *prec_rounding))
    def __neg__(s): return mpc(-s.real, -s.imag)
    def __nonzero__(s): return bool(s.real) or bool(s.imag)
    def conjugate(s): return mpc(s.real, -s.imag)

    def __eq__(s, t):
        if not isinstance(t, mpc):
            if isinstance(t, str):
                return False
            t = mpc_convert_lhs(t)
            if t is NotImplemented:
                return t
        return s.real == t.real and s.imag == t.imag

    def __ne__(s, t):
        b = s.__eq__(t)
        if b is NotImplemented:
            return b
        return not b

    def _compare(*args):
        raise TypeError("no ordering relation is defined for complex numbers")

    __gt__ = _compare
    __le__ = _compare
    __gt__ = _compare
    __ge__ = _compare

    def __add__(s, t):
        prec, rounding = prec_rounding
        if not isinstance(t, mpc):
            t = mpc_convert_lhs(t)
            if t is NotImplemented:
                return t
            if isinstance(t, mpf):
                return make_mpc(mpc_add_mpf(s._mpc_, t._mpf_, prec, rounding))
        return make_mpc(mpc_add(s._mpc_, t._mpc_, prec, rounding))

    def __sub__(s, t):
        prec, rounding = prec_rounding
        if not isinstance(t, mpc):
            t = mpc_convert_lhs(t)
            if t is NotImplemented:
                return t
            if isinstance(t, mpf):
                return make_mpc(mpc_sub_mpf(s._mpc_, t._mpf_, prec, rounding))
        return make_mpc(mpc_sub(s._mpc_, t._mpc_, prec, rounding))

    def __mul__(s, t):
        prec, rounding = prec_rounding
        if not isinstance(t, mpc):
            if isinstance(t, int_types):
                return make_mpc(mpc_mul_int(s._mpc_, t, prec, rounding))
            t = mpc_convert_lhs(t)
            if t is NotImplemented:
                return t
            if isinstance(t, mpf):
                return make_mpc(mpc_mul_mpf(s._mpc_, t._mpf_, prec, rounding))
            t = mpc(t)
        return make_mpc(mpc_mul(s._mpc_, t._mpc_, prec, rounding))

    def __div__(s, t):
        prec, rounding = prec_rounding
        if not isinstance(t, mpc):
            t = mpc_convert_lhs(t)
            if t is NotImplemented:
                return t
            if isinstance(t, mpf):
                return make_mpc(mpc_div_mpf(s._mpc_, t._mpf_, prec, rounding))
        return make_mpc(mpc_div(s._mpc_, t._mpc_, prec, rounding))

    def __pow__(s, t):
        prec, rounding = prec_rounding
        if isinstance(t, int_types):
            return make_mpc(mpc_pow_int(s._mpc_, t, prec, rounding))
        t = mpc_convert_lhs(t)
        if t is NotImplemented:
            return t
        if isinstance(t, mpf):
            return make_mpc(mpc_pow_mpf(s._mpc_, t._mpf_, prec, rounding))
        return make_mpc(mpc_pow(s._mpc_, t._mpc_, prec, rounding))

    __radd__ = __add__

    def __rsub__(s, t):
        t = mpc_convert_lhs(t)
        if t is NotImplemented:
            return t
        return t - s

    def __rmul__(s, t):
        prec, rounding = prec_rounding
        if isinstance(t, int_types):
            return make_mpc(mpc_mul_int(s._mpc_, t, prec, rounding))
        t = mpc_convert_lhs(t)
        if t is NotImplemented:
            return t
        return t * s

    def __rdiv__(s, t):
        t = mpc_convert_lhs(t)
        if t is NotImplemented:
            return t
        return t / s

    def __rpow__(s, t):
        t = mpc_convert_lhs(t)
        if t is NotImplemented:
            return t
        return t ** s

    __truediv__ = __div__
    __rtruediv__ = __rdiv__

    def ae(s, t, rel_eps=None, abs_eps=None):
        return almosteq(s, t, rel_eps, abs_eps)


complex_types = (complex, mpc)


class mpi(mpnumeric):
    """
    Interval arithmetic class. Precision is controlled by mp.prec.
    """

    def __new__(cls, a, b=None):
        if isinstance(a, mpi):
            return a
        if b is None:
            b = a
        a = mpf(a, rounding=round_floor)
        b = mpf(b, rounding=round_ceiling)
        if isnan(a) or isnan(b):
            a, b = -inf, inf
        assert a <= b, "endpoints must be properly ordered"
        return make_mpi((a._mpf_, b._mpf_))

    @property
    def a(self):
        return make_mpf(self._val[0])

    @property
    def b(self):
        return make_mpf(self._val[1])

    @property
    def mid(self):
        return make_mpf(mpi_mid(self._val, mp.prec))

    @property
    def delta(self):
        return make_mpf(mpi_delta(self._val, mp.prec))

    def _compare(*args):
        raise TypeError("no ordering relation is defined for intervals")

    __gt__ = _compare
    __le__ = _compare
    __gt__ = _compare
    __ge__ = _compare

    def __contains__(self, t):
        t = mpi(t)
        return (self.a <= t.a) and (t.b <= self.b)

    def __repr__(self):
        return mpi_str(self._val, mp.prec)

    __str__ = __repr__

    def __eq__(self, other):
        if not isinstance(other, mpi):
            try:
                other = mpi(other)
            except:
                return NotImplemented
        return (self.a == other.a) and (self.b == other.b)

    def __abs__(self):
        return make_mpi(mpi_abs(self._val, mp.prec))

    def __pos__(self):
        return make_mpi(mpi_pos(self._val, mp.prec))

    def __neg__(self):
        return make_mpi(mpi_neg(self._val, mp.prec))

    def __add__(self, other):
        if not isinstance(other, mpi):
            other = mpi(other)
        return make_mpi(mpi_add(self._val, other._val, mp.prec))

    def __sub__(self, other):
        if not isinstance(other, mpi):
            other = mpi(other)
        return make_mpi(mpi_sub(self._val, other._val, mp.prec))

    def __mul__(self, other):
        if not isinstance(other, mpi):
            other = mpi(other)
        return make_mpi(mpi_mul(self._val, other._val, mp.prec))

    def __div__(self, other):
        if not isinstance(other, mpi):
            other = mpi(other)
        return make_mpi(mpi_div(self._val, other._val, mp.prec))

    def __pow__(self, other):
        if isinstance(other, (int, long)):
            return make_mpi(mpi_pow_int(self._val, int(other), mp.prec))
        if not isinstance(other, mpi):
            other = mpi(other)
        return make_mpi(mpi_pow(self._val, other._val, mp.prec))

    def __rsub__(s, t):
        return mpi(t) - s

    def __rdiv__(s, t):
        return mpi(t) / s

    def __rpow__(s, t):
        return mpi(t) ** s

    __radd__ = __add__
    __rmul__ = __mul__
    __truediv__ = __div__
    __rtruediv__ = __rdiv__
    __floordiv__ = __div__
    __rfloordiv__ = __rdiv__

def make_mpi(val, cls=mpi):
    a = new(cls)
    a._val = val
    return a

def make_mpf(v, cls=mpf):
    a = new(cls)
    a._mpf_ = v
    return a

def make_mpc(v, cls=mpc):
    a = new(cls)
    a._mpc_ = v
    return a

one = make_mpf(fone)
zero = make_mpf(fzero)
inf = make_mpf(finf)
ninf = make_mpf(fninf)
nan = make_mpf(fnan)
j = mpc(0,1)

class constant(mpf):
    """Represents a mathematical constant with dynamic precision.
    When printed or used in an arithmetic operation, a constant
    is converted to a regular mpf at the working precision. A
    regular mpf can also be obtained using the operation +x."""

    def __new__(cls, func, name):
        a = object.__new__(cls)
        a.name = name
        a.func = func
        return a

    def __call__(self, prec=None, dps=None, rounding=None):
        if not prec: prec = prec_rounding[0]
        if not rounding: rounding = prec_rounding[1]
        if dps: prec = dps_to_prec(prec)
        return make_mpf(self.func(prec, rounding))

    @property
    def _mpf_(self):
        prec, rounding = prec_rounding
        return self.func(prec, rounding)

    def __repr__(self):
        return "<%s: %s~>" % (self.name, nstr(self))

eps = constant(lambda prec, rnd: (0, MP_ONE, 1-prec, 1),
    "epsilon of working precision")


def rand():
    """Return an mpf chosen randomly from [0, 1)."""
    return make_mpf(mpf_rand(mp.prec))

def isnan(x):
    if not isinstance(x, mpf):
        return False
    return x._mpf_ == fnan

def isinf(x):
    if not isinstance(x, mpf):
        return False
    return x._mpf_ in (finf, fninf)

def isint(x):
    if isinstance(x, int_types):
        return True
    try:
        x = convert_lossless(x)
    except:
        return False
    if isinstance(x, mpf):
        if isnan(x) or isinf(x):
            return False
        return x == int(x)
    return False

def absmin(x):
    """
    Returns abs(x).a for an interval, or abs(x) for anything else.
    """
    if isinstance(x, mpi):
        return abs(x).a
    return abs(x)

def absmax(x):
    """
    Returns abs(x).b for an interval, or abs(x) for anything else.
    """
    if isinstance(x, mpi):
        return abs(x).b
    return abs(x)

def AS_POINTS(x):
    if isinstance(x, mpi):
        return [x.a, x.b]
    return x

def fraction(p, q):
    """Given Python integers p, q, return a lazy mpf with value p/q.
    The value is updated with the precision.

        >>> mp.dps = 15
        >>> a = fraction(1,100)
        >>> b = mpf(1)/100
        >>> print a; print b
        0.01
        0.01
        >>> mp.dps = 30
        >>> print a; print b
        0.01
        0.0100000000000000002081668171172
        >>> mp.dps = 15
    """
    return constant(lambda prec, rnd: from_rational(p, q, prec, rnd),
        '%s/%s' % (p, q))

from operator import gt, lt

def arange(*args):
    """arange([a,] b[, dt]) -> list [a, a + dt, a + 2*dt, ..., x] with x < b"""
    if not len(args) <= 3:
        raise TypeError('arange expected at most 3 arguments, got %i'
                        % len(args))
    if not len(args) >= 1:
        raise TypeError('arange expected at least 1 argument, got %i'
                        % len(args))
    # set default
    a = 0
    dt = 1
    # interpret arguments
    if len(args) == 1:
        b = args[0]
    elif len(args) >= 2:
        a = args[0]
        b = args[1]
    if len(args) == 3:
        dt = args[2]
    a, b, dt = mpf(a), mpf(b), mpf(dt)
    assert a + dt != a, 'dt is too small and would cause an infinite loop'
    # adapt code for sign of dt
    if a > b:
        if dt > 0:
            return []
        op = gt
    else:
        if dt < 0:
            return []
        op = lt
    # create list
    result = []
    i = 0
    t = a
    while 1:
        t = a + dt*i
        i += 1
        if op(t, b):
            result.append(t)
        else:
            break
    return result

def linspace(*args, **kwargs):
    """
    linspace(a, b, n, [endpoint=False]) -> n evenly spaced samples from a to b

    Instead of a, b you can specify an mpi interval.
    """
    if len(args) == 3:
        a = mpf(args[0])
        b = mpf(args[1])
        n = int(args[2])
    elif len(args) == 2:
        assert isinstance(args[0], mpi)
        a = args[0].a
        b = args[0].b
        n = int(args[1])
    else:
        raise TypeError('linspace expected 2 or 3 arguments, got %i' \
                        % len(args))
    if n < 1:
        raise ValueError('n must be greater than 0')
    if not 'endpoint' in kwargs or kwargs['endpoint']:
        if n == 1:
            return [mpf(a)]
        step = (b - a) / mpf(n - 1)
        y = [i*step + a for i in xrange(n)]
        y[-1] = b
    else:
        step = (b - a) / mpf(n)
        y = [i*step + a for i in xrange(n)]
    return y

def almosteq(s, t, rel_eps=None, abs_eps=None):
    """
    Determine whether the difference between s and t is smaller
    than a given epsilon.

    Both a maximum relative difference and a maximum difference
    ('epsilons') may be specified. The absolute difference is
    defined as |s-t| and the relative difference is defined
    as |s-t|/max(|s|, |t|).

    If only one epsilon is given, both are set to the same value.
    If none is given, both epsilons are set to 2**(-prec+m) where
    prec is the current working precision and m is a small integer.
    """
    t = convert_lossless(t)
    if abs_eps is None and rel_eps is None:
        rel_eps = abs_eps = make_mpf((0, 1, -mp.prec+4, 1))
    if abs_eps is None:
        abs_eps = rel_eps
    elif rel_eps is None:
        rel_eps = abs_eps
    diff = abs(s-t)
    if diff <= abs_eps:
        return True
    abss = abs(s)
    abst = abs(t)
    if abss < abst:
        err = diff/abst
    else:
        err = diff/abss
    return err <= rel_eps

def nstr(x, n=6):
    """Convert an mpf or mpc to a decimal string literal with n significant
    digits. The small default value for n is chosen to make this function
    useful for printing collections of numbers.

    If x is a list or tuple, the function is applied to each element.
    For unrecognized classes, this simply returns str(x).

    There is also a companion function nprint that prints the string
    instead of returning it.

        >>> nstr([+pi, ldexp(1,-500)])
        '[3.14159, 3.05494e-151]'
        >>> print([+pi, ldexp(1,-500)])
        [3.14159, 3.05494e-151]
    """
    if isinstance(x, list):
        return "[%s]" % (", ".join(nstr(c, n) for c in x))
    if isinstance(x, tuple):
        return "(%s)" % (", ".join(nstr(c, n) for c in x))
    if isinstance(x, mpf):
        return to_str(x._mpf_, n)
    if isinstance(x, mpc):
        return "(" + complex_to_str(x._mpc_[0], x._mpc_[1], n)  + ")"
    if isinstance(x, basestring):
        return repr(x)
    from matrices import matrix
    if isinstance(x, matrix):
        return x.__nstr__(n)
    return str(x)

def nprint(x, n=6):
    """Print the result of nstr(x, n)."""
    print nstr(x, n)



"""
High-level, mostly calculus-oriented functions.

* Numerical differentiation
* Numerical polynomial operations
* Numerical root-finding
* Numerical integration

etc
"""

__docformat__ = 'plaintext'

from settings import (mp, extraprec)
from mptypes import (mpnumeric, convert_lossless, mpf, mpc, j, inf, eps,
    AS_POINTS, arange, nstr)
from functions import (ldexp, factorial, exp, cos, pi, bernoulli)

from quadrature import quadgl, quadts

#----------------------------------------------------------------------------#
#                       General extrapolation methods                        #
#----------------------------------------------------------------------------#

def richardson_extrapolation(f, n, N):
    if not callable(f):
        g = f; f = lambda k: g.__getitem__(int(k))
    orig = mp.prec
    try:
        mp.prec = 2*orig
        s = mpf(0)
        for j in range(0, N+1):
            c = (n+j)**N * (-1)**(j+N) / (factorial(j) * factorial(N-j))
            s += c * f(mpf(n+j))
    finally:
        mp.prec = orig
    return +s

def shanks_extrapolation(f, n, m=1):
    if not callable(f):
        g = f; f = lambda k: g.__getitem__(int(k))
    orig = mp.prec
    try:
        mp.prec = 2*orig
        table = [f(mpf(j)) for j in range(n+m+2)]
        table2 = table[:]
        for i in range(1, m+1):
            for j in range(i, n+m+1):
                x, y, z = table[j-1], table[j], table[j+1]
                try:
                    table2[j] = (z*x - y**2) / (z + x - 2*y)
                except ZeroDivisionError:
                    return table2[j-1]
            table = table2[:]
    finally:
        mp.prec = orig
    return +table[n]

def limit(f, x, direction=-1, n=None, N=None):
    """Compute lim of f(t) as t -> x using Richardson extrapolation.
    For infinite x, the function values [f(n), ... f(n+N)] are used.
    For finite x, [f(x-direction/n)), ... f(x-direction/(n+N))] are
    used. If x is inf, f can be also be a precomputed sequence
    with a __getitem__ method."""
    if callable(f):
        if not n: n = 3 + int(mp.dps * 0.5)
        if not N: N = 2*n
    else:
        # If a sequence, take as many terms are are available
        g = f; f = lambda k: g.__getitem__(int(k))
        if not N: N = len(g)-1
        if not n: n = 0
    if   x == inf:  return richardson_extrapolation(lambda k: f(mpf(k)), n, N)
    elif x == -inf: return richardson_extrapolation(lambda k: f(mpf(-k)), n, N)
    direction *= mpf(1)
    def g(k):
        return f(x - direction/(1+k))
    return richardson_extrapolation(g, n, N)


#----------------------------------------------------------------------------#
#                                Differentiation                             #
#----------------------------------------------------------------------------#

def diff(f, x, direction=0):
    """
    Compute f'(x) using a simple finite difference approximation.

    With direction = 0, use the central difference f(x-h), f(x+h)
    With direction = 1, use the forward difference f(x), f(x+h)
    With direction = -1, use the backward difference f(x-h), f(x)

        >>> print diff(cos, 1)
        -0.841470984807897
        >>> print diff(abs, 0, 0)
        0.0
        >>> print diff(abs, 0, 1)
        1.0
        >>> print diff(abs, 0, -1)
        -1.0

    The step size is taken similar to the epsilon of the precision.
    To eliminate cancellation errors, diff temporarily doubles the
    working precision while calculating the function values.
    """
    prec = mp.prec
    extra = 5
    h = ldexp(1, -prec-extra)
    try:
        mp.prec = 2*(prec+extra)
        if   direction == 0:  return (f(x+h) - f(x-h)) * ldexp(1, prec+extra-1)
        elif direction == 1:  return (f(x+h) - f(x)) * ldexp(1, prec+extra)
        elif direction == -1: return (f(x) - f(x-h)) * ldexp(1, prec+extra)
        else:
            raise ValueError("invalid difference direction: %r" % direction)
    finally:
        mp.prec = prec

def diffc(f, x, n=1, radius=mpf(0.5)):
    """
    Compute an approximation of the nth derivative of f at the point x
    using the Cauchy integral formula. This only works for analytic
    functions. A circular path with the given radius is used.

    diffc increases the working precision slightly to avoid simple
    rounding errors. Note that, especially for large n, differentiation
    is extremely ill-conditioned, so this precaution does not
    guarantee a correct result. (Provided there are no singularities
    in the way, increasing the radius may help.)

    The returned value will be a complex number; a large imaginary part
    for a derivative that should be real may indicate a large numerical
    error.
    """
    prec = mp.prec
    try:
        mp.prec += 10
        def g(t):
            rei = radius*exp(j*t)
            z = x + rei
            return f(z) / rei**n
        d = quadts(g, [0, 2*pi])
        return d * factorial(n) / (2*pi)
    finally:
        mp.prec = prec


#----------------------------------------------------------------------------#
#                           Generic root-finding                             #
#----------------------------------------------------------------------------#

msg1 = "Cannot perform a step with the secant method because the " \
  "function values are equal at the two chosen start points. Try " \
  "different start points."

msg2 = "The derivative cannot be computed. The previous value " \
  "will be reused for the next iteration."

def secant(f, x0, x1=None, maxsteps=20, verbose=False):
    """Solve the equation f(x) = 0 using the secant method, starting
    at the given initial point x0 and performing up to `maxsteps`
    steps or quitting when the difference between successive x values
    is smaller than the epsilon of the current working precision.

    The secant method requires a second starting point x1 with both
    x0 and x1 located close to the root. If only x0 is provided, x1
    is automatically generated as x0 + 1/4."""
    weps = 2*eps
    x = x0 * mpf(1)
    if x1 is None:
        xprev = x0 + mpf(0.25)
    else:
        xprev = x1 * mpf(1)
    deriv_prev = None
    fxprev = f(xprev)
    for i in xrange(maxsteps):
        if verbose:
            print "Step", i
            print "x =", x
        fx = f(x)
        ydiff = fx - fxprev
        xdiff = x - xprev
        if verbose:
            print "f(x) =", fx
            print "xdiff = ", xdiff
            print "ydiff = ", ydiff
        try:
            deriv = xdiff / ydiff
            deriv_prev = deriv
        except ZeroDivisionError:
            if deriv_prev is None:
                raise ZeroDivisionError(msg1)
            if verbose and abs(xdiff) > weps:
                print msg2
            deriv = deriv_prev
        x, xprev = x - fx*deriv, x
        fxprev = fx
        if verbose:
            print
        if abs(xdiff) <= weps:
            break
    return x


#----------------------------------------------------------------------------#
#                                Polynomials                                 #
#----------------------------------------------------------------------------#

def polyval(coeffs, x, derivative=False):
    """
    Given coefficients [cn, ..., c2, c1, c0], evaluate
    P(x) = cn*x**n + ... + c2*x**2 + c1*x + c0.

    If derivative=True is set, a tuple (P(x), P'(x)) is returned.
    """
    if not coeffs:
        return mpf(0)
    p = mpnumeric(coeffs[0])
    q = mpf(0)
    for c in coeffs[1:]:
        if derivative:
            q = p + x*q
        p = c + x*p
    if derivative:
        return p, q
    else:
        return p

def polyroots(coeffs, maxsteps=50, cleanup=True, extraprec=10, error=False):
    """
    Numerically locate all (complex) roots of a polynomial using the
    Durand-Kerner method.

    With error=True, this function returns a tuple (roots, err) where roots
    is a list of complex numbers sorted by absolute value, and err is an
    estimate of the maximum error. The polynomial should be given as a list
    of coefficients, in the same format as accepted by polyval(). The
    leading coefficient must be nonzero.

    These are the roots of x^3 - x^2 - 14*x + 24 and 4x^2 + 3x + 2:

        >>> nprint(polyroots([1,-1,-14,24]), 4)
        [-4.0, 2.0, 3.0]
        >>> nprint(polyroots([4,3,2], error=True))
        ([(-0.375 - 0.599479j), (-0.375 + 0.599479j)], 2.22045e-16)

    """
    if len(coeffs) <= 1:
        if not coeffs or not coeffs[0]:
            raise ValueError("Input to polyroots must not be the zero polynomial")
        # Constant polynomial with no roots
        return []

    orig = mp.prec
    weps = +eps
    try:
        mp.prec += 10
        deg = len(coeffs) - 1
        # Must be monic
        lead = convert_lossless(coeffs[0])
        if lead == 1:
            coeffs = map(convert_lossless, coeffs)
        else:
            coeffs = [c/lead for c in coeffs]
        f = lambda x: polyval(coeffs, x)
        roots = [mpc((0.4+0.9j)**n) for n in xrange(deg)]
        err = [mpf(1) for n in xrange(deg)]
        for step in xrange(maxsteps):
            if max(err).ae(0):
                break
            for i in xrange(deg):
                if not err[i].ae(0):
                    p = roots[i]
                    x = f(p)
                    for j in range(deg):
                        if i != j:
                            try:
                                x /= (p-roots[j])
                            except ZeroDivisionError:
                                continue
                    roots[i] = p - x
                    err[i] = abs(x)
        if cleanup:
            for i in xrange(deg):
                if abs(roots[i].imag) < weps:
                    roots[i] = roots[i].real
                elif abs(roots[i].real) < weps:
                    roots[i] = roots[i].imag * 1j
        roots.sort(key=lambda x: (abs(x.imag), x.real))
    finally:
        mp.prec = orig
    if error:
        err = max(err)
        err = max(err, ldexp(1, -orig+1))
        return [+r for r in roots], +err
    else:
        return [+r for r in roots]

def quadosc(f, interval, period=None, zeros=None, alt=1):
    """
    Integrates f(x) over interval = [a, b] where at least one of a and
    b is infinite and f is a slowly decaying oscillatory function. The
    zeros of f must be provided, either by specifying a period (suitable
    when f contains a pure sine or cosine factor) or providing a function
    that returns the nth zero (suitable when the oscillation is not
    strictly periodic).
    """
    a, b = AS_POINTS(interval)
    a = convert_lossless(a)
    b = convert_lossless(b)
    if period is None and zeros is None:
        raise ValueError( \
            "either the period or zeros keyword parameter must be specified")
    if a == -inf and b == inf:
        s1 = quadosc(f, [a, 0], zeros=zeros, period=period, alt=alt)
        s2 = quadosc(f, [0, b], zeros=zeros, period=period, alt=alt)
        return s1 + s2
    if a == -inf:
        if zeros:
            return quadosc(lambda x:f(-x), [-b,-a], lambda n: zeros(-n), alt=alt)
        else:
            return quadosc(lambda x:f(-x), [-b,-a], period=period, alt=alt)
    if b != inf:
        raise ValueError("quadosc requires an infinite integration interval")
    if not zeros:
        zeros = lambda n: n*period/2
    for n in range(1,10):
        p = zeros(n)
        if p > a:
            break
    if n >= 9:
        raise ValueError("zeros do not appear to be correctly indexed")
    if alt == 0:
        s = quadgl(f, [a, zeros(n+1)])
        s += sumrich(lambda k: quadgl(f, [zeros(2*k), zeros(2*k+2)]), [n, inf])
    else:
        s = quadgl(f, [a, zeros(n)])
        s += sumsh(lambda k: quadgl(f, [zeros(k), zeros(k+1)]), [n, inf])
    return s


##############################################################################
##############################################################################

#----------------------------------------------------------------------------#
#                               Numerical summation                          #
#----------------------------------------------------------------------------#

def sumrich(f, interval, n=None, N=None):
    """
    Sum f(k) for k = a, a+1, ..., b where [a, b] = interval,
    using Richardson extrapolation. This function is essentially
    equivalent to using limit() on the sequence of partial sums.
    """
    a, b = AS_POINTS(interval)
    assert b == inf
    if not n: n = 3 + int(mp.dps * 0.5)
    if not N: N = 2*n
    orig = mp.prec
    try:
        mp.prec = 2*orig
        s = mpf(0)
        tbl = []
        for k in range(a, a+n+N+1):
            s += f(mpf(k))
            tbl.append(s)
        s = richardson_extrapolation(lambda k: tbl[int(k)], n, N)
    finally:
        mp.prec = orig
    return +s

def sumsh(f, interval, n=None, m=None):
    """
    Sum f(k) for k = a, a+1, ..., b where [a, b] = interval,
    using an n-term Shanks transformation. With m > 1, the Shanks
    transformation is applied recursively m times.

    Shanks summation often works well for slowly convergent and/or
    alternating Taylor series.
    """
    a, b = AS_POINTS(interval)
    assert b == inf
    if not n: n = 5 + int(mp.dps * 1.2)
    if not m: m = 2 + n//3
    orig = mp.prec
    try:
        mp.prec = 2*orig
        s = mpf(0)
        tbl = []
        for k in range(a, a+n+m+2):
            s += f(mpf(k))
            tbl.append(s)
        s = shanks_extrapolation(tbl, n, m)
    finally:
        mp.prec = orig
    return +s

@extraprec(15, normalize_output=True)
def sumem(f, interval, N=None, integral=None, fderiv=None, error=False,
    verbose=False):
    """
    Sum f(k) for k = a, a+1, ..., b where [a, b] = interval,
    using Euler-Maclaurin summation. This algorithm is efficient
    for slowly convergent nonoscillatory sums; the essential condition
    is that f must be analytic. The method relies on approximating the
    sum by an integral, so f must be smooth and well-behaved enough
    to be integrated numerically.

    With error=True, a tuple (s, err) is returned where s is the
    calculated sum and err is the estimated magnitude of the error.
    With verbose=True, detailed information about progress and errors
    is printed.

        >>> mp.dps = 15
        >>> s, err = sumem(lambda n: 1/n**2, 1, inf, error=True)
        >>> print s
        1.64493406684823
        >>> print pi**2 / 6
        1.64493406684823
        >>> nprint(err)
        2.22045e-16

    N is the number of terms to compute directly before using the
    Euler-Maclaurin formula to approximate the tail. It must be set
    high enough; often roughly N ~ dps is the right size.

    High-order derivatives of f are also needed. By default, these
    are computed using numerical integration, which is the most
    expensive part of the calculation. The default method assumes
    that all poles of f are located close to the origin. A custom
    nth derivative function fderiv(x, n) can be provided as a
    keyword parameter.

    This is much more efficient:

        >>> f = lambda n: 1/n**2
        >>> fp = lambda x, n: (-1)**n * factorial(n+1) * x**(-2-n)
        >>> mp.dps = 50
        >>> print sumem(lambda n: 1/n**2, 1, inf, fderiv=fp)
        1.6449340668482264364724151666460251892189499012068
        >>> print pi**2 / 6
        1.6449340668482264364724151666460251892189499012068

    If b = inf, f and its derivatives are all assumed to vanish
    at infinity. It is assumed that a is finite, so doubly
    infinite sums cannot be evaluated directly.
    """
    a, b = AS_POINTS(interval)
    if N is None:
        N = 3*mp.dps + 20
    a, b, N = mpf(a), mpf(b), mpf(N)
    infinite = (b == inf)
    weps = eps * 2**8
    if verbose:
        print "Summing f(k) from k = %i to %i" % (a, a+N-1)
    S = sum(f(mpf(k)) for k in xrange(a, a+N))
    if integral is None:
        if verbose:
            print "Integrating f(x) from x = %i to %s" % (a+N, nstr(b))
        I, ierr = quadts(f, [a+N, b], error=1)
        # XXX: hack for relative error
        ierr /= abs(I)
    else:
        I, ierr = integral(a+N, b), mpf(0)
    # There is little hope if the tail cannot be integrated
    # accurately. Estimate magnitude of tail as the error.
    if ierr > weps:
        if verbose:
            print "Failed to converge to target accuracy (integration failed)"
        if error:
            return S+I, abs(I) + ierr
        else:
            return S+I
    if infinite:
        C = f(a+N) / 2
    else:
        C = (f(a+N) + f(b)) / 2
    # Default (inefficient) approach for derivatives
    if not fderiv:
        fderiv = lambda x, n: diffc(f, x, n, radius=N*0.75)
    k = 1
    prev = 0
    if verbose:
        print "Summing tail"
    fac = 2
    while 1:
        if infinite:
            D = fderiv(a+N, 2*k-1)
        else:
            D = fderiv(a+N, 2*k-1) - fderiv(b, 2*k-1)
        # B(2*k) / fac(2*k)
        term = bernoulli(2*k) / fac * D
        mag = abs(term)
        if verbose:
            print "term", k, "magnitude =", nstr(mag)
        # Error can be estimated as the magnitude of the smallest term
        if k >= 2:
            if mag < weps:
                if verbose:
                    print "Converged to target accuracy"
                res, err = I + C + S, eps * 2**15
                break
            if mag > abs(prev):
                if verbose:
                    print "Failed to converge to target accuracy (N too low)"
                res, err = I + C + S, abs(term)
                break
        S -= term
        k += 1
        fac *= (2*k) * (2*k-1)
        prev = term
    if isinstance(res, mpc) and not isinstance(I, mpc):
        res, err = res.real, err
    if error:
        return res, err
    else:
        return res

#----------------------------------------------------------------------------#
#                                  ODE solvers                               #
#----------------------------------------------------------------------------#

def smul(a, x):
    """Multiplies the vector "x" by the scalar "a"."""
    R = []
    for i in range(len(x)):
        R.append(a*x[i])
    return R

def vadd(*args):
    """Adds vectors "x", "y", ... together."""
    assert len(args) >= 2
    n = len(args[0])
    rest = args[1:]
    for x in args:
        assert len(x) == n
    R = []
    for i in range(n):
        s = args[0][i]
        for x in rest:
            s += x[i]
        R.append(s)
    return R

def ODE_step_euler(x, y, h, derivs):
    """
    Advances the solution y(x) from x to x+h using the Euler method.

    derivs .... a python function f(x, (y1, y2, y3, ...)) returning
    a tuple (y1', y2', y3', ...) where y1' is the derivative of y1 at x.
    """
    X = derivs(y,x)
    return vadd(y, smul(h, X))

half = mpf(0.5)

def ODE_step_rk4(x, y, h, derivs):
    """
    Advances the solution y(x) from x to x+h using the 4th-order Runge-Kutta
    method.

    derivs .... a python function f(x, (y1, y2, y3, ...)) returning
    a tuple (y1', y2', y3', ...) where y1' is the derivative of y1 at x.
    """
    h2 = ldexp(h, -1)
    third = mpf(1)/3
    k1 = smul(h, derivs(y, x))
    k2 = smul(h, derivs(vadd(y, smul(half, k1)), x+h2))
    k3 = smul(h, derivs(vadd(y, smul(half, k2)), x+h2))
    k4 = smul(h, derivs(vadd(y, k3), x+h))
    v = []
    for i in range(len(y)):
        v.append(y[i] + third*(k2[i]+k3[i] + half*(k1[i]+k4[i])))
    return v

def odeint(derivs, x0, t_list, step=ODE_step_rk4):
    """
    Given the list t_list of values, returns the solution at these points.
    """
    x = x0
    result = [x]
    for i in range(len(t_list)-1):
        dt = t_list[i+1] - t_list[i]
        x = step(t_list[i], x, dt, derivs)
        result.append(x)
    return result

#----------------------------------------------------------------------------#
#                              Approximation methods                         #
#----------------------------------------------------------------------------#

# The Chebyshev approximation formula is given at:
# http://mathworld.wolfram.com/ChebyshevApproximationFormula.html

# The only major changes in the following code is that we return the
# expanded polynomial coefficients instead of Chebyshev coefficients,
# and that we automatically transform [a,b] -> [-1,1] and back
# for convenience.

# Coefficient in Chebyshev approximation
def chebcoeff(f,a,b,j,N):
    s = mpf(0)
    h = mpf(0.5)
    for k in range(1, N+1):
        t = cos(pi*(k-h)/N)
        s += f(t*(b-a)*h + (b+a)*h) * cos(pi*j*(k-h)/N)
    return 2*s/N

# Generate Chebyshev polynomials T_n(ax+b) in expanded form
def chebT(a=1, b=0):
    Tb = [1]
    yield Tb
    Ta = [b, a]
    while 1:
        yield Ta
        # Recurrence: T[n+1](ax+b) = 2*(ax+b)*T[n](ax+b) - T[n-1](ax+b)
        Tmp = [0] + [2*a*t for t in Ta]
        for i, c in enumerate(Ta): Tmp[i] += 2*b*c
        for i, c in enumerate(Tb): Tmp[i] -= c
        Ta, Tb = Tmp, Ta

def chebyfit(f, interval, N, error=False):
    """
    Chebyshev approximation: returns coefficients of a degree N-1
    polynomial that approximates f on the interval [a, b]. With error=True,
    also returns an estimate of the maximum error.
    """
    a, b = AS_POINTS(interval)
    orig = mp.prec
    try:
        mp.prec = orig + int(N**0.5) + 20
        c = [chebcoeff(f,a,b,k,N) for k in range(N)]
        d = [mpf(0)] * N
        d[0] = -c[0]/2
        h = mpf(0.5)
        T = chebT(mpf(2)/(b-a), mpf(-1)*(b+a)/(b-a))
        for k in range(N):
            Tk = T.next()
            for i in range(len(Tk)):
                d[i] += c[k]*Tk[i]
        d = d[::-1]
        # Estimate maximum error
        err = mpf(0)
        for k in range(N):
            x = cos(pi*k/N) * (b-a)*h + (b+a)*h
            err = max(err, abs(f(x) - polyval(d, x)))
    finally:
        mp.prec = orig
        if error:
            return d, +err
        else:
            return d
